#include "StdAfx.h"
#include "OutputDebugStream.h"
#include "SSDSmart.h"
#include "JSAlert.h"
#include <sstream>
#include <iomanip>
#include "resource.h"
#include <assert.h>
#include <boost/assign.hpp>

using namespace std;
using namespace boost::assign;
extern HINSTANCE g_hInst;

CJSAlert::CJSAlert(void)
{
	_hWnd = NULL;
	_bFirst = true;

	_vBalloonAlerts += "Good Block Count";
	_vDialogAlerts  += "ECC Fail Count", "Program Failure Block Count", "Erase Failure Block Count", 
										 "Read Failure Block Count", "Bad Block Full Flag", "Read Error Rate";
	_vAllAlerts += "Average Erase Count","Maximum Erase Count","Good Block Count","System Block Count","ECC Fail Count",
								 "Minimum Erase Count","Read Error Rate","Program Failure Block Count",
								 "Erase Failure Block Count","Read Failure Block Count","Total Error Bits Count","Read Sectors with CBE Count","Bad Block Full Flag";
}

CJSAlert::~CJSAlert(void){
	DWORD dwParam;
	for (std::list<HANDLE>::const_iterator it = _lDialogs.begin(); it != _lDialogs.end(); ++it) {
		if (GetExitCodeThread(*it , &dwParam) && (dwParam == STILL_ACTIVE)) {
			TerminateThread(*it, FALSE);
		}
	}
}

CJSAlert &CJSAlert::GetInstance() {
	static CJSAlert Instance;
	return Instance;
}

static wstring g_sTitle;
static wstring g_sMsg;

extern HWND g_hWnd;
DWORD WINAPI ThreadFunc(LPVOID vdParam) {
	MessageBox(NULL, g_sMsg.c_str(), g_sTitle.c_str(), MB_ICONEXCLAMATION | MB_SETFOREGROUND);
	return 0;
}

void CJSAlert::ShowAlert(EAlertType eType, const std::wstring &rTitle, const std::wstring &rMessage) {

	if (eType == eBalloon) {
		NOTIFYICONDATA	sData;
		ZeroMemory(&sData, sizeof(NOTIFYICONDATA));
		sData.cbSize		= sizeof(NOTIFYICONDATA);
		sData.uFlags		= NIF_INFO;
		sData.hWnd			= _hWnd;
		sData.dwInfoFlags	= NIIF_INFO;
		sData.uTimeout		= 3*1000;
		lstrcpy(sData.szInfoTitle, rTitle.c_str());
		lstrcpy(sData.szInfo, rMessage.c_str());
		Shell_NotifyIcon(NIM_MODIFY,&sData);
	}
	if (eType == eDialog) {
		g_sTitle = rTitle;
		g_sMsg = rMessage;
		DWORD dwID;
		_lDialogs.push_back(CreateThread(NULL, 0, ThreadFunc, NULL , 0 ,&dwID));
	}
}

void CJSAlert::SetLastAttributes(const std::vector<boost::shared_ptr<CJSStatus> > &rvLast) {
	//̊֐ĂłȂNotify̒ŏ
	_bFirst = false;
	_vLastAttribute = rvLast;
}

void CJSAlert::Notify(const CJSSmart& rSubject) {

	if (_bFirst) {
		_vLastAttribute = rSubject.GetSSDStatus();
		_bFirst = false;
	}
	else {
		for (size_t i=0; i<rSubject.GetSuitableDriveNum(); ++i) {
			string sResult1 = rSubject.GetSSDStatus()[i]->GetStatusChangeString(_vLastAttribute[i].get(), _vBalloonAlerts);
			string sResult2 = rSubject.GetSSDStatus()[i]->GetStatusChangeString(_vLastAttribute[i].get(), _vDialogAlerts);
			if (sResult1 != "") {
				ShowAlert(eBalloon, _T("JSMonitor Alert"), widen(sResult1));
				_vLastAttribute[i] = rSubject.GetSSDStatus()[i];
			}
			if (sResult2 != "") {
				ShowAlert(eDialog, _T("JSMonitor Alert"), widen(sResult2));
				_vLastAttribute[i] = rSubject.GetSSDStatus()[i];
			}
		}
	}
}


