#include "StdAfx.h"
#include "JSIDXStatus.h"
#include "Resource.h"
#include <sstream>
#include <iomanip>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/tokenizer.hpp>
#include <boost/lexical_cast.hpp>
using namespace std;

void CJSIDXStatus::AddStatusDetailString(std::ostream &os, bool bHideSerial) const {

	double dDriveSize = _IDXStatus.nDriveSize/1000.0/1000.0/1000.0;
	os << "<Drive #" << _IDXStatus.nDeviceNo << ">" << "\r\n";
	os << "Controller:                  " << "Indilinx Barefoot" << "\r\n";
	os << "Model:                       " << _IDXStatus.sModelName << "\r\n";
	os << "Drive Size:                  " << setprecision(1) << fixed  << dDriveSize << "GB\r\n";
	if (!bHideSerial) os << "Serial Number:               " << _IDXStatus.sSerialNo << "\r\n";
	os << "Firmware Version:            " << _IDXStatus.sFirmwareVersion << "\r\n";

	os << "Raw Read Error Rate:         " << _IDXStatus.nRawReadErrorRate << "\r\n";
	os << "Power On Hours:              " << _IDXStatus.nPoweronHours << "\r\n";
	os << "Power Cycle Count:           " << _IDXStatus.nDevicePowerCycleCount << "\r\n";
	os << "Average Erase Count:         " << _IDXStatus.nAverageEraseCount << "\r\n";
	os << "Maximum Erase Count:         " << _IDXStatus.nMaxEraseCount << "\r\n";
	os << "Minimum Erase Count:         " << _IDXStatus.nMinEraseCount << "\r\n";
	os << "Initial Bad Block Count:     " << _IDXStatus.nInitialBadBlockCount << "\r\n";
	os << "Program Failure Block Count: " << _IDXStatus.nProgramFailureBlockCount << "\r\n";
	os << "Erase Failure Block Count:   " << _IDXStatus.nEraseFailureBlockCount << "\r\n";
	os << "Read Failure Block Count:    " << _IDXStatus.nReadFailureBlockCount << "\r\n";
	os << "Total Sectors Read:          " << _IDXStatus.nTotalCountofReadSectors << "\r\n";
	os << "Total Sectors Written:       " << _IDXStatus.nTotalCountofWriteSectors << "\r\n";
	os << "Total Read Commands:         " << _IDXStatus.nTotalCountofReadCommands << "\r\n";
	os << "Total Write Commands:        " << _IDXStatus.nTotalCountofWriteCommands << "\r\n";
	os << "Total Error Bits Count:      " << _IDXStatus.nTotalCountofErrorBitsfromFlash << "\r\n";
	os << "Total Sectors Read with CBE: " << _IDXStatus.nTotalCountofReadSectorswithCorrectableBitErrors << "\r\n";
	os << "Bad Block Full Flag:         " << _IDXStatus.nBadBlockFullFlag << "\r\n";
	os << "Maximum PE Count Spec:       " << _IDXStatus.nMaximumPECountSpecification << "\r\n";
	os << "Remaining Drive Life:        " << _IDXStatus.nRemainingDriveLife << "[%]\r\n";

}

void CJSIDXStatus::UpdateSSDDialog(std::vector<HWND>& rvSmartDialogs, bool bHideSerial) const {

	HWND hDlgIDX = rvSmartDialogs[1];
	ShowWindow(hDlgIDX, SW_SHOWNORMAL);

	SetDlgItemULLWithComma(hDlgIDX, IDC_AVERAGE_ERASE, _IDXStatus.nAverageEraseCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_MAX_ERASE, _IDXStatus.nMaxEraseCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_MINIMUM_ERASE, _IDXStatus.nMinEraseCount);
	SetDlgItemTextA(hDlgIDX, IDC_FIRMWARE, _IDXStatus.sFirmwareVersion.c_str());
	SetDlgItemULLWithComma(hDlgIDX, IDC_POWER_CYCLE, _IDXStatus.nDevicePowerCycleCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_POWER_ON_HOURS, _IDXStatus.nPoweronHours);
	SetDlgItemULLWithComma(hDlgIDX, IDC_READ_ERROR, _IDXStatus.nRawReadErrorRate);
	SetDlgItemULLWithComma(hDlgIDX, IDC_INITIAL_BAD, _IDXStatus.nInitialBadBlockCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_PE_FAILURE, _IDXStatus.nProgramFailureBlockCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_ERASE_FAILURE, _IDXStatus.nEraseFailureBlockCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_READ_FAILURE, _IDXStatus.nReadFailureBlockCount);
	SetDlgItemULLWithComma(hDlgIDX, IDC_ERROR_BITS, _IDXStatus.nTotalCountofErrorBitsfromFlash);
	SetDlgItemULLWithComma(hDlgIDX, IDC_READ_SECTORS_CBE, _IDXStatus.nTotalCountofReadSectorswithCorrectableBitErrors);
	SetDlgItemULLWithComma(hDlgIDX, IDC_MAXIMUM_PE_SPEC, _IDXStatus.nMaximumPECountSpecification);
	SetDlgItemULLWithComma(hDlgIDX, IDC_BAD_BLOCK_FULL, _IDXStatus.nBadBlockFullFlag);
	SetDlgItemULLWithComma(hDlgIDX, IDC_READ_SECTOR, _IDXStatus.nTotalCountofReadSectors);
	SetDlgItemULLWithComma(hDlgIDX, IDC_READ_COMMAND, _IDXStatus.nTotalCountofReadCommands);
	SetDlgItemULLWithComma(hDlgIDX, IDC_WRITE_SECTOR, _IDXStatus.nTotalCountofWriteSectors);
	SetDlgItemULLWithComma(hDlgIDX, IDC_WRITE_COMMAND, _IDXStatus.nTotalCountofWriteCommands);
	if (bHideSerial) SetDlgItemTextA(hDlgIDX, IDC_SERIALNO, "**********");
	else SetDlgItemTextA(hDlgIDX, IDC_SERIALNO, _IDXStatus.sSerialNo.c_str());
}

bool CJSIDXStatus::IsUpdated(const CJSStatus* pStatus) const {

	const CIDXStatus *pTarget = dynamic_cast<const CIDXStatus*>(pStatus->GetStatus());
	if (pTarget &&
			(_IDXStatus.nAverageEraseCount == pTarget->nAverageEraseCount) &&
			(_IDXStatus.nMaxEraseCount == pTarget->nMaxEraseCount) &&
			(_IDXStatus.nMinEraseCount == pTarget->nMinEraseCount) &&
			(_IDXStatus.nRawReadErrorRate == pTarget->nRawReadErrorRate) &&
			(_IDXStatus.nProgramFailureBlockCount == pTarget->nProgramFailureBlockCount) &&
			(_IDXStatus.nEraseFailureBlockCount == pTarget->nEraseFailureBlockCount) &&
			(_IDXStatus.nReadFailureBlockCount == pTarget->nReadFailureBlockCount) &&
			(_IDXStatus.nBadBlockFullFlag == pTarget->nBadBlockFullFlag) &&
			(_IDXStatus.nTotalCountofErrorBitsfromFlash == pTarget->nTotalCountofErrorBitsfromFlash) &&
			(_IDXStatus.nTotalCountofReadSectorswithCorrectableBitErrors == pTarget->nTotalCountofReadSectorswithCorrectableBitErrors)
			)
			return false;
	else return true;
	return false;
}

std::string CJSIDXStatus::WriteToCSV() const {
	std::ostringstream os;
	os << _IDXStatus.tMeasuredTime << "," << _IDXStatus.nAverageEraseCount << "," << _IDXStatus.nMaxEraseCount << ",";
	os << _IDXStatus.nMinEraseCount << "," << _IDXStatus.nDevicePowerCycleCount << "," << _IDXStatus.nPoweronHours << ",";
	os << _IDXStatus.nRawReadErrorRate << "," << _IDXStatus.nInitialBadBlockCount << ",";
	os << _IDXStatus.nProgramFailureBlockCount << "," << _IDXStatus.nEraseFailureBlockCount << "," << _IDXStatus.nReadFailureBlockCount << ",";
	os << _IDXStatus.nTotalCountofErrorBitsfromFlash << "," << _IDXStatus.nTotalCountofReadSectorswithCorrectableBitErrors << ",";
	os << _IDXStatus.nMaximumPECountSpecification << "," << _IDXStatus.nBadBlockFullFlag << ",";
	os << _IDXStatus.nTotalCountofReadSectors << "," << _IDXStatus.nTotalCountofReadCommands << ",";
	os << _IDXStatus.nTotalCountofWriteSectors << "," << _IDXStatus.nTotalCountofWriteCommands;
	return os.str();
}

bool CJSIDXStatus::ReadFromCSV(const std::string &rsLine) {

	try {
		boost::char_separator<char> sep(",");
		boost::tokenizer<boost::char_separator<char> > Tokens(rsLine, sep);

		boost::tokenizer<boost::char_separator<char> >::iterator tok_iter = Tokens.begin();
		if (tok_iter == Tokens.end()) return false;
		_IDXStatus.tMeasuredTime = boost::lexical_cast<boost::posix_time::ptime>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nAverageEraseCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nMaxEraseCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nMinEraseCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nDevicePowerCycleCount = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nPoweronHours = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nRawReadErrorRate = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nInitialBadBlockCount = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nProgramFailureBlockCount = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nEraseFailureBlockCount = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nReadFailureBlockCount = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nTotalCountofErrorBitsfromFlash = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nTotalCountofReadSectorswithCorrectableBitErrors = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nMaximumPECountSpecification = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nBadBlockFullFlag = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nTotalCountofReadSectors = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nTotalCountofReadCommands = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nTotalCountofWriteSectors = boost::lexical_cast<unsigned long long>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_IDXStatus.nTotalCountofWriteCommands = boost::lexical_cast<unsigned long long>(*tok_iter);
	}
	catch (std::exception &e) {
		dout << __FUNCTION__ << e.what() << endl;
		return false;
	}
	return true;
}

std::string CJSIDXStatus::MakeCSVHeader() const {

	ostringstream oss;
	oss << "Barefoot JSMonitor Log v1.0" << endl;
	oss << "Date-Time" << "," << "Average Erase Count"<< ","  << "Maximum Erase Count"<< "," ;
	oss << "Minimum Erase Count" << "," << "Power Cycle Count" << "," << "Power on Hours" << ",";
	oss << "Raw Read Error Rate" << "," << "Initial Bad Block Count" << "," ;
	oss << "Program Failure Block Count" << "," << "Erase Failure Block Count" << "," << "Read Failure Block Count" << "," ;
	oss << "Total Count of Error Bits from Flash" << "," << "Total Count of Read Sectors with Correctable Bit Errors" << "," ;
	oss << "Maximum PE Count Specification" << "," << "Bad Block Full Flag" << "," ;
	oss << "Total Count of Read Sectors" << "," << "Total Count of Read Commands" << "," ;
	oss << "Total Count of Write Sectors" << "," << "Total Count of Write Commands";
	return oss.str();
}
unsigned long long CJSIDXStatus::GetValueWithKeyword(const std::string &rKey) const{
	if (rKey == "Average Erase Count") return _IDXStatus.nAverageEraseCount;
	if (rKey == "Maximum Erase Count") return _IDXStatus.nMaxEraseCount;
	if (rKey == "Minimum Erase Count") return _IDXStatus.nMinEraseCount;
	if (rKey == "Read Error Rate") return _IDXStatus.nRawReadErrorRate;
	if (rKey == "Program Failure Block Count") return _IDXStatus.nProgramFailureBlockCount;
	if (rKey == "Erase Failure Block Count") return _IDXStatus.nEraseFailureBlockCount;
	if (rKey == "Read Failure Block Count") return _IDXStatus.nReadFailureBlockCount;
	if (rKey == "Total Error Bits Count") return _IDXStatus.nTotalCountofErrorBitsfromFlash;
	if (rKey == "Read Sectors with CBE Count") return _IDXStatus.nTotalCountofReadSectorswithCorrectableBitErrors;
	if (rKey == "Bad Block Full Flag") return _IDXStatus.nBadBlockFullFlag;
	return 0;
}

#include <boost/assign.hpp>

std::string CJSIDXStatus::GetStatusChangeString(const CJSStatus* pOldStatus, const std::vector<string> &vTargetStrings) const {

	using namespace boost::assign;
	ostringstream osMsg;
	bool bChanged = false;
	osMsg << "Drive #" << _IDXStatus.nDeviceNo << " (" << _IDXStatus.sModelName << ")\n";
	const CJSIDXStatus* pOldStatus2 = dynamic_cast<const CJSIDXStatus*>(pOldStatus);
	if (!pOldStatus2) return "";

	vector<string> vKeyCandidates;
	vKeyCandidates += "Average Erase Count","Maximum Erase Count","Minimum Erase Count","Read Error Rate","Program Failure Block Count",
										"Erase Failure Block Count","Read Failure Block Count","Total Error Bits Count","Read Sectors with CBE Count","Bad Block Full Flag";

	for (size_t i=0; i<vKeyCandidates.size(); ++i) {
		if (std::find(vTargetStrings.begin(), vTargetStrings.end(), vKeyCandidates[i]) != vTargetStrings.end()) {
			unsigned long long n1 = GetValueWithKeyword(vKeyCandidates[i]);
			unsigned long long n2 = pOldStatus2->GetValueWithKeyword(vKeyCandidates[i]);
			if (n1 != n2) {
				osMsg << vKeyCandidates[i] << " Changed. Old:" << n2 << " Current:" << n1 << "\n";
				bChanged = true;
			}
		}
	}
	if (bChanged) return osMsg.str();
	else return "";
}


boost::shared_ptr<CJSStatus> CJSIDXStatus::MakeClone() const {
	return boost::shared_ptr<CJSStatus>(new CJSIDXStatus(*this));
}

int CJSIDXStatus::GetEraseCountMax() const {

	return (int)round(_IDXStatus.nMaximumPECountSpecification*(CJSStatus::_dIDXMultiply));
}
