#include "StdAfx.h"
#include "JSJMFStatus.h"
#include "Resource.h"
#include <sstream>
#include <iomanip>
#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/tokenizer.hpp>
#include <boost/lexical_cast.hpp>
using namespace std;

void CJSJMFStatus::AddStatusDetailString(std::ostream &os, bool bHideSerial) const {

	double dDriveSize = _JMFStatus.nDriveSize/1000.0/1000.0/1000.0;
	os << "<Drive #" << _JMFStatus.nDeviceNo << ">" << "\r\n";
	os << "Controller:                  " << "JMicron JMF601/602 Family" << "\r\n";
	os << "Model:                       " << _JMFStatus.sModelName << "\r\n";
	os << "Drive Size:                  " << setprecision(1) << fixed  << dDriveSize << "GB\r\n";
	if (!bHideSerial) os << "Serial Number:               " << _JMFStatus.sSerialNo << "\r\n";
	os << "Firmware Version:            " << _JMFStatus.sFirmwareVersion << "\r\n";

	os << setprecision(6);
	os << "Firmware Date:               " << _JMFStatus.sFirmwareDate << "\r\n";
	os << "Flash Vendor:                " << _JMFStatus.sFlashVendor << "\r\n";
	if ( _JMFStatus.nFlashType == CSSDStatus::eMLC) 
		os << "Flash Type:                  " << "MLC" << "\r\n";
	else if (_JMFStatus.nFlashType == CSSDStatus::eSLC) 
		os << "Flash Type:                  " << "SLC" << "\r\n";
	else 
		os << "Flash Type:                  " << "Unknown" << "\r\n";
	os << "Flash Name:                  " << _JMFStatus.sFlashName << "\r\n";
	os << "Block Size:                  " << _JMFStatus.nBlockSize << "MB\r\n";
	os << "Flash ID:                    ";
	os << hex << uppercase;
	for (size_t j=0; j<7; ++j) {
		os << "0x" << setw(2) << setfill('0') << (int)_JMFStatus.anFlashID[j+1] << " ";
	}
	os << "\r\n";
	os << dec;

	os << "Channels:                    " << _JMFStatus.nChannels << "\r\n";
	os << "Banks:                       " << _JMFStatus.nBanks << "\r\n";
	os << "Power Cycle Count:           " << _JMFStatus.nPowerCycleCount << "\r\n";
	os << "Average Erase Count:         " << _JMFStatus.nAverageEraseCount << "\r\n";
	os << "Maximum Erase Count:         " << _JMFStatus.nMaxEraseCount << "\r\n";
	os << "Good Block Count:            " << _JMFStatus.nGoodBlockCount << "\r\n";
	os << "System Block Count:          " << _JMFStatus.nSystemBlockCount << "\r\n";
	os << "Error Correction Count:      " << _JMFStatus.nECCFailCount << "\r\n";
	os << "Last EC Channel:             " << (int)_JMFStatus.nLastECChannel << "\r\n";
	os << "Last EC Bank:                " << (int)_JMFStatus.nLastECBank << "\r\n";
	os << "Last EC Address:             ";
	os << hex << uppercase;
	for (size_t j=0; j<3; ++j) {
		os << "0x" << setw(2) << setfill('0') << (int)_JMFStatus.anLastECAddress[j] << " ";
	}
	os << "\r\n";
	os << dec;
}

void CJSJMFStatus::UpdateSSDDialog(std::vector<HWND>& rvSmartDialogs, bool bHideSerial) const {
	
	for (size_t i=0; i<rvSmartDialogs.size(); ++i) {
		ShowWindow(rvSmartDialogs[i], SW_HIDE);
	}
	HWND hDlgJMF = rvSmartDialogs[0];
	ShowWindow(hDlgJMF, SW_SHOWNORMAL);
	SetDlgItemULLWithComma(hDlgJMF, IDC_AVERAGE_ERASE, _JMFStatus.nAverageEraseCount);
	SetDlgItemULLWithComma(hDlgJMF, IDC_MAX_ERASE, _JMFStatus.nMaxEraseCount);
	SetDlgItemULLWithComma(hDlgJMF, IDC_GOOD_BLOCK, _JMFStatus.nGoodBlockCount);
	SetDlgItemULLWithComma(hDlgJMF, IDC_SYSTEM_BLOCK, _JMFStatus.nSystemBlockCount);
	SetDlgItemULLWithComma(hDlgJMF, IDC_POWER_CYCLE, _JMFStatus.nPowerCycleCount);
	SetDlgItemULLWithComma(hDlgJMF, IDC_ERROR_COUNT, _JMFStatus.nECCFailCount);
	SetDlgItemULLWithComma(hDlgJMF, IDC_ECBANK, _JMFStatus.nLastECBank);
	SetDlgItemULLWithComma(hDlgJMF, IDC_ECCHANNEL, _JMFStatus.nLastECChannel);

	ostringstream osECC;
	osECC << hex << uppercase;
	for (size_t i=0; i<3; ++i) {
		if (i!=0) osECC << " ";
		osECC << "0x" << setw(2) << setfill('0') << (int)_JMFStatus.anLastECAddress[i];
	}
	SetDlgItemTextA(hDlgJMF, IDC_ECADDRESS, osECC.str().c_str());
	SetDlgItemTextA(hDlgJMF, IDC_FIRMWARE,     _JMFStatus.sFirmwareVersion.c_str());
	SetDlgItemTextA(hDlgJMF, IDC_FIRMWAREDATE, _JMFStatus.sFirmwareDate.c_str());
	SetDlgItemTextA(hDlgJMF, IDC_FLASHVENDOR,  _JMFStatus.sFlashVendor.c_str());
	SetDlgItemTextA(hDlgJMF, IDC_FLASHNAME, _JMFStatus.sFlashName.c_str());
	SetDlgItemInt(hDlgJMF, IDC_BANKS, _JMFStatus.nBanks, true);
	SetDlgItemInt(hDlgJMF, IDC_CHANNELS, _JMFStatus.nChannels, true);

	ostringstream osFlashID;
	osFlashID << hex << uppercase;
	for (size_t i=0; i<7; ++i) {
		if (i!=0) osFlashID << " ";
		osFlashID << "0x" << setw(2) << setfill('0') << (int)_JMFStatus.anFlashID[i+1];
	}
	SetDlgItemTextA(hDlgJMF, IDC_FLASHID, osFlashID.str().c_str());
//			sprintf_s(temp, 10, "%dMB", _JMFStatus.nBlockSize);

	if (bHideSerial) SetDlgItemTextA(hDlgJMF, IDC_SERIALNO, "**********");
	else SetDlgItemTextA(hDlgJMF, IDC_SERIALNO, _JMFStatus.sSerialNo.c_str());

	if (_JMFStatus.nFlashType == CSSDStatus::eMLC) SetDlgItemTextA(hDlgJMF, IDC_FLASHTYPE, "MLC");
	else if (_JMFStatus.nFlashType == CSSDStatus::eSLC) SetDlgItemTextA(hDlgJMF, IDC_FLASHTYPE, "SLC");
	else SetDlgItemTextA(hDlgJMF, IDC_FLASHTYPE, "Unknown");

}

bool CJSJMFStatus::IsUpdated(const CJSStatus* pStatus) const {

	const CJMFStatus *pTarget = dynamic_cast<const CJMFStatus*>(pStatus->GetStatus());
	if (pTarget &&
			(_JMFStatus.nAverageEraseCount == pTarget->nAverageEraseCount) &&
			(_JMFStatus.nMaxEraseCount == pTarget->nMaxEraseCount) &&
			(_JMFStatus.nGoodBlockCount == pTarget->nGoodBlockCount) &&
			(_JMFStatus.nSystemBlockCount == pTarget->nSystemBlockCount) &&
			(_JMFStatus.nECCFailCount == pTarget->nECCFailCount))
			return false;
	else return true;
	return false;
}

std::string CJSJMFStatus::WriteToCSV() const {
	std::ostringstream os;
	os << _JMFStatus.tMeasuredTime << "," << _JMFStatus.nPowerCycleCount << "," << _JMFStatus.nAverageEraseCount << "," << _JMFStatus.nMaxEraseCount << ",";
	os << _JMFStatus.nGoodBlockCount << "," << _JMFStatus.nSystemBlockCount << ",";
	os << _JMFStatus.nECCFailCount << "," << _JMFStatus.nLastECChannel << "," << _JMFStatus.nLastECBank << ",";
	os << (int)_JMFStatus.anLastECAddress[0] << "," << (int)_JMFStatus.anLastECAddress[1] << "," << (int)_JMFStatus.anLastECAddress[2]; 
	return os.str();
}

bool CJSJMFStatus::ReadFromCSV(const std::string &rsLine) {

	try {
		boost::char_separator<char> sep(",");
		boost::tokenizer<boost::char_separator<char> > Tokens(rsLine, sep);

		boost::tokenizer<boost::char_separator<char> >::iterator tok_iter = Tokens.begin();
		if (tok_iter == Tokens.end()) return false;
		_JMFStatus.tMeasuredTime = boost::lexical_cast<boost::posix_time::ptime>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nPowerCycleCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nAverageEraseCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nMaxEraseCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nGoodBlockCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nSystemBlockCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nECCFailCount = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nLastECChannel = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.nLastECBank = boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.anLastECAddress[0] = (BYTE)boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.anLastECAddress[1] = (BYTE)boost::lexical_cast<unsigned int>(*tok_iter);
		++tok_iter; if (tok_iter == Tokens.end()) return false;
		_JMFStatus.anLastECAddress[2] = (BYTE)boost::lexical_cast<unsigned int>(*tok_iter);
	}
	catch (std::exception &e) {
		dout << __FUNCTION__ << e.what() << endl;
		return false;
	}
	return true;
}

std::string CJSJMFStatus::MakeCSVHeader() const {

	ostringstream oss;
	oss << "JMF601/602 JSMonitor Log v1.0" << endl;
	oss << "Date-Time" << "," << "Power Cycle Count" << "," << "Average Erase Count" << "," << "Maximum Erase Count" << ",";
	oss << "Good Block Count" << "," << "System Block Count" << ",";
	oss << "ECC Fail Count" << "," << "ECC Fail Channel" << "," << "ECC Fail Bank" << ",";
	oss << "ECC Fail Address1" << "," << "ECC Fail Address2" << "," << "ECC Fail Address3";
	return oss.str();
}

std::string CJSJMFStatus::GetStatusChangeString(const CJSStatus* pOldStatus, const std::vector<string> &vTargetStrings) const {

	ostringstream osMsg;
	bool bChanged = false;
	osMsg << "Drive #" << _JMFStatus.nDeviceNo << " (" << _JMFStatus.sModelName << ")\n";
	const CJSJMFStatus* pOldStatus2 = dynamic_cast<const CJSJMFStatus*>(pOldStatus);
	const CJMFStatus &rOldStatus = pOldStatus2->_JMFStatus;
	if (std::find(vTargetStrings.begin(), vTargetStrings.end(), "Average Erase Count") != vTargetStrings.end()) {
		if (_JMFStatus.nAverageEraseCount != rOldStatus.nAverageEraseCount) {
			osMsg << "Average Erase Count Changed. Old:" << rOldStatus.nAverageEraseCount << " Current:" << _JMFStatus.nAverageEraseCount << "\n";
			bChanged = true;
		}
	}
	if (std::find(vTargetStrings.begin(), vTargetStrings.end(), "Maximum Erase Count") != vTargetStrings.end()) {
		if (_JMFStatus.nMaxEraseCount != rOldStatus.nMaxEraseCount) {
			osMsg << "Maximum Erase Count Changed. Old:" << rOldStatus.nMaxEraseCount << " Current:" << _JMFStatus.nMaxEraseCount << "\n";
			bChanged = true;
		}
	}
	if (std::find(vTargetStrings.begin(), vTargetStrings.end(), "Good Block Count") != vTargetStrings.end()) {
		if (_JMFStatus.nGoodBlockCount != rOldStatus.nGoodBlockCount) {
			osMsg << "Good Block Count Changed. Old:" << rOldStatus.nGoodBlockCount << " Current:" << _JMFStatus.nGoodBlockCount << "\n";
			bChanged = true;
		}
	}
	if (std::find(vTargetStrings.begin(), vTargetStrings.end(), "System Block Count") != vTargetStrings.end()) {
		if (_JMFStatus.nSystemBlockCount != rOldStatus.nSystemBlockCount) {
			osMsg << "System Block Count Changed. Old:" << rOldStatus.nSystemBlockCount << " Current:" << _JMFStatus.nSystemBlockCount << "\n";
			bChanged = true;
		}
	}
	if (std::find(vTargetStrings.begin(), vTargetStrings.end(), "ECC Fail Count") != vTargetStrings.end()) {
		if (_JMFStatus.nECCFailCount != rOldStatus.nECCFailCount) {
			osMsg << "ECC Fail Count Changed. Old:" << rOldStatus.nECCFailCount << " Current:" << _JMFStatus.nECCFailCount << "\n";
			osMsg << "Channel: " << _JMFStatus.nLastECChannel << " Bank: " << _JMFStatus.nLastECBank << " Address: ";
			for (size_t j=0; j<3; ++j) {
				osMsg << " 0x" << setw(2) << setfill('0') << (int)_JMFStatus.anLastECAddress[j];
			}
			bChanged = true;
		}
	}
	if (bChanged) {
		return osMsg.str();
	}
	else return "";
}


boost::shared_ptr<CJSStatus> CJSJMFStatus::MakeClone() const {
	return boost::shared_ptr<CJSStatus>(new CJSJMFStatus(*this));
}

int CJSJMFStatus::GetEraseCountMax() const {

	if (_JMFStatus.nFlashType == CSSDStatus::eSLC) {
		return _nSLCEraseCountMax;
	}
	else {
		return _nMLCEraseCountMax;
	}
}

bool CJSJMFStatus::ReadStatusFromOldLog (std::istream &is) {

	CJMFStatus BackUp = _JMFStatus;

	is >> _JMFStatus.tMeasuredTime;
	is >> _JMFStatus.nPowerCycleCount >> _JMFStatus.nAverageEraseCount >> _JMFStatus.nMaxEraseCount >> _JMFStatus.nGoodBlockCount >> _JMFStatus.nSystemBlockCount;
	int tmp;
	is >> tmp >> _JMFStatus.nECCFailCount;
	for (int i=0; i<3; ++i) {
		is >> tmp;
		_JMFStatus.anLastECAddress[i] = (BYTE)tmp;
	}
	is >> _JMFStatus.nLastECChannel >> _JMFStatus.nLastECBank;

	if ((_JMFStatus.tMeasuredTime != BackUp.tMeasuredTime) && 
			(
				(_JMFStatus.nAverageEraseCount != BackUp.nAverageEraseCount) ||
				(_JMFStatus.nMaxEraseCount != BackUp.nMaxEraseCount) ||
				(_JMFStatus.nGoodBlockCount != BackUp.nGoodBlockCount) ||
				(_JMFStatus.nSystemBlockCount != BackUp.nSystemBlockCount) ||
				(_JMFStatus.nECCFailCount != BackUp.nECCFailCount)
				)) 
	{
		return true;
	}
	dout << "Read Failed!" << endl;
	return false;
}

