#include "StdAfx.h"
#include "JSLogger.h"
#include "OutputDebugStream.h"
#include "LifeSpanEstimation.h"
#include <shlwapi.h>
#include "JSJMFStatus.h"

using namespace std;
using namespace boost;
using namespace boost::posix_time;
using namespace boost::gregorian;

const boost::gregorian::date& GetEpoch() {
	static date dEpoch(2008, Jan, 1);
	return dEpoch;
}

double ptime2date(const ptime &rTime) {
	date dDate = rTime.date();
	time_duration dTime = rTime.time_of_day();
	date dEpoch(2008, Jan, 1);
	date_duration dSa = dDate-GetEpoch();
	return dSa.days()+dTime.hours()/24.0+dTime.minutes()/(24.0*60.0)+dTime.seconds()/(24.0*60.0*60.0);
};

ptime date2ptime(double dDate) {
	double d0 = dDate-(int)(dDate);
	int nHour = (int)(24.0*d0);
	double d1 = 24.0*d0-nHour;
	int nMinute = (int)(60.0*d1);
	double d2 = 60.0*d1-nMinute;
	int nSecond = (int)(floor((60.0*d2)+0.5));
	date dEpoch(2008, Jan, 1);
	return ptime(GetEpoch()+date_duration((int)dDate), time_duration(nHour, nMinute, nSecond));
}

CJSLogger::CJSLogger(void){
}

CJSLogger::~CJSLogger(void){
}

CJSLogger &CJSLogger::GetInstance() {
	static CJSLogger Instance;
	return Instance;
}

void CJSLogger::Notify(const CJSSmart& rSubject) {
	if (!_bLogOpened) ReadLog(rSubject);
	else UpdateLog(rSubject);
}

wstring MakeFileName(const CJSStatus& rStatus) {
	const CSSDStatus *pSSDStatus = rStatus.GetStatus();
	wstringstream sFileName;
	sFileName << GetModulePath() << "\\" << widen(pSSDStatus->sModelName) << "_" << widen(pSSDStatus->sSerialNo) << ".csv";
	return sFileName.str();
}

wstring MakeOldFileName(const CJSStatus& rStatus) {
	const CSSDStatus *pSSDStatus = rStatus.GetStatus();
	wstringstream sFileName;
	sFileName << GetModulePath() << "\\" << widen(pSSDStatus->sModelName) << "_" << widen(pSSDStatus->sSerialNo) << ".log";
	return sFileName.str();
}

std::pair<const CLifeSpanEstimation::SLifeSpanResult *, const std::vector<int> *> CJSLogger::GetLSEResult(size_t nDriveNum, bool bUseMax) const{

	if (_vbNeedRecalcLSE[nDriveNum]) {
		int nMaxNum = _vpLastSSDStatus[nDriveNum]->GetEraseCountMax();
		_vsAveResult[nDriveNum] = _LSE.GetLifeSpan(_vDateTimeLog[nDriveNum], _vnAveEraseLog[nDriveNum], nMaxNum, &_vnAveUsedList[nDriveNum]);
		_vsMaxResult[nDriveNum] = _LSE.GetLifeSpan(_vDateTimeLog[nDriveNum], _vnMaxEraseLog[nDriveNum], nMaxNum, &_vnMaxUsedList[nDriveNum]);
		_vbNeedRecalcLSE[nDriveNum] = false;
	}
	if (bUseMax) {
		return make_pair(&_vsMaxResult[nDriveNum],&_vnMaxUsedList[nDriveNum]);
	}
	else {
		return make_pair(&_vsAveResult[nDriveNum],&_vnAveUsedList[nDriveNum]);
	}
}

//蓮ŃOύXۂɉsYꂽ̑΍
bool bNeadLF(ifstream &ifs) {
	ifs.clear();
	ifs.seekg(-2, ios::end);
	char a[1];
	ifs.read(a, 1);
	return (a[0] != '\n');
}

bool CJSLogger::ReadFromOldLog(const CJSStatus* pStatus) {

	wstring sOldFileName = MakeOldFileName(*pStatus);
	ifstream ifs(sOldFileName.c_str());
	boost::shared_ptr<CJSStatus> pStatus2 = pStatus->MakeClone();
	CJSJMFStatus* pJMFStatus = dynamic_cast<CJSJMFStatus*>(pStatus2.get());
	dout << "read from old" << endl;
	dout << ifs.is_open() << " " << pJMFStatus << endl;
	if (ifs.is_open() && pJMFStatus) {
		dout << "Old Log Found!" << endl;
		wstring sFileName = MakeFileName(*pStatus);
		ofstream ofs(sFileName.c_str());
		ofs << pStatus2->MakeCSVHeader() << endl;
		string sLine; ptime Ptime;
		while (getline(ifs, sLine)) {
			istringstream iss(sLine);
			if (pJMFStatus->ReadStatusFromOldLog(iss)) {
				ofs << pJMFStatus->WriteToCSV() << endl;
				dout << pJMFStatus->WriteToCSV() << endl;
			}
		}
		ofs.close();
		ofs.clear();
		return true;
	}
	return false;
}

void CJSLogger::ReadLog(const CJSSmart& rSubject) {

	_vbNeedRecalcLSE.resize(rSubject.GetSuitableDriveNum());
	_vsMaxResult.resize(rSubject.GetSuitableDriveNum());
	_vsAveResult.resize(rSubject.GetSuitableDriveNum());
	_vnMaxUsedList.resize(rSubject.GetSuitableDriveNum());
	_vnAveUsedList.resize(rSubject.GetSuitableDriveNum());

	_vnMaxEraseLog.resize(rSubject.GetSuitableDriveNum());
	_vnAveEraseLog.resize(rSubject.GetSuitableDriveNum());
	_vDateTimeLog.resize(rSubject.GetSuitableDriveNum());

	const std::vector<boost::shared_ptr<CJSStatus> > &rvStatus = rSubject.GetSSDStatus();
	for (size_t i=0; i<rvStatus.size(); ++i) {
		_vbNeedRecalcLSE[i] = true;
		boost::shared_ptr<CJSStatus> pStatus = rvStatus[i]->MakeClone();
		_vDateTimeLog.push_back( vector<SDate>() );
		vector<unsigned int> &rAveLog = _vnAveEraseLog[i];
		vector<unsigned int> &rMaxLog = _vnMaxEraseLog[i];
		vector<SDate> &rDateLog = _vDateTimeLog[i];
		bool bNeedLF = false;
		bool bNeedHeader = true;
		wstring sFileName = MakeFileName(*pStatus);
		if (!PathFileExists(sFileName.c_str())) {
			ReadFromOldLog(pStatus.get());
		}
		ifstream ifs(sFileName.c_str());
		if (ifs.is_open()) {
			double dLastDayFromEpoch = 0; string sLine;
			getline(ifs, sLine); //header1
			getline(ifs, sLine); //header2
			while (getline(ifs, sLine)) {
				if (pStatus->ReadFromCSV(sLine)) {
					SDate day(pStatus->GetStatus()->tMeasuredTime);
					if (dLastDayFromEpoch > day.dDayFromEpoch) {
						dout << "LogFile Time Broken! old= " << dLastDayFromEpoch << " now= " << day.dDayFromEpoch << endl;
					}
					else {
						dout << "ReadLog pushback: " << pStatus->WriteToCSV() << endl;
						rAveLog.push_back(pStatus->GetStatus()->nAverageEraseCount);
						rMaxLog.push_back(pStatus->GetStatus()->nMaxEraseCount);
						rDateLog.push_back(day);
						dLastDayFromEpoch = day.dDayFromEpoch;
					}
				}
			}
			bNeedLF=bNeadLF(ifs);
			bNeedHeader = false;
			ifs.close();
			ifs.clear();
		}

		dout << "LastStatus: " << pStatus->WriteToCSV() << endl;
		_vpLastSSDStatus.push_back(pStatus);
		shared_ptr<ofstream> pFile(new ofstream(sFileName.c_str(), ios::out|ios::app));
		if (bNeedHeader) (*pFile) << pStatus->MakeCSVHeader() << endl;
		if (bNeedLF) (*pFile) << endl;
		_vLogFiles.push_back(pFile);
		_vSSDType.push_back(pStatus->GetStatus()->nFlashType);
	}
	_bLogOpened = true;
}


void CJSLogger::UpdateLog(const CJSSmart& rSubject) {

	for (size_t i=0; i<rSubject.GetSuitableDriveNum(); ++i) {
		const boost::shared_ptr<CJSStatus> pStatus = rSubject.GetSSDStatus()[i];
		if (pStatus->IsUpdated(_vpLastSSDStatus[i].get()) || _vDateTimeLog[i].empty()) {
			_vbNeedRecalcLSE[i] = true;
			const CSSDStatus* pCurrentStatus = pStatus->GetStatus();
			const CSSDStatus* pLastStatus    = _vpLastSSDStatus[i]->GetStatus();
			if (pCurrentStatus->tMeasuredTime < pLastStatus->tMeasuredTime) {
				ostringstream oss;
				oss << "Clock Broken! Current Time = " << pCurrentStatus->tMeasuredTime << " Last Time = " << pLastStatus->tMeasuredTime << endl;
				throw(std::exception(oss.str().c_str()));
			}
			SDate CurrentDate(pCurrentStatus->tMeasuredTime);
			_vDateTimeLog[i].push_back(CurrentDate);
			_vnAveEraseLog[i].push_back(pCurrentStatus->nAverageEraseCount);
			_vnMaxEraseLog[i].push_back(pCurrentStatus->nMaxEraseCount);
			(*_vLogFiles[i]) << pStatus->WriteToCSV() << endl;
			_vpLastSSDStatus[i] = pStatus->MakeClone();
		}
	}
}
