#include "StdAfx.h"
#include "JSSmart.h"
#include "JSJMFStatus.h"
#include "JSIDXStatus.h"


int CJSStatus::_nSLCEraseCountMax = 100000;
int CJSStatus::_nMLCEraseCountMax = 10000;
double CJSStatus::_dIDXMultiply = 1.0;


boost::shared_ptr<CJSStatus> MakeJSStatus(CSSDStatus* pStatus) {
	CJMFStatus* pJMF = dynamic_cast<CJMFStatus*>(pStatus);
	if (pJMF) {
		return boost::shared_ptr<CJSStatus> (new CJSJMFStatus(pJMF));
	}
	CIDXStatus* pIDX = dynamic_cast<CIDXStatus*>(pStatus);
	if (pIDX) {
		return boost::shared_ptr<CJSStatus> (new CJSIDXStatus(pIDX));
	}
	return boost::shared_ptr<CJSStatus>();
}

CJSSmart::CJSSmart(CSSDSmart *pSmartImpl) {
	_pSmartImpl=pSmartImpl; 
	Update();
}

CJSSmart::~CJSSmart() {
}

CJSSmart &CJSSmart::GetInstance() {
	static CJSSmart Instance(&CSSDSmart::GetInstance());
	return Instance;
}

extern HWND g_hWnd;
void CJSSmart::Update() {
	try {
		_pSmartImpl->Update();
		_vpSSDStatus.clear();
		for (size_t i=0; i<_pSmartImpl->GetSSDStatus().size(); ++i) {
			_vpSSDStatus.push_back(MakeJSStatus(_pSmartImpl->GetSSDStatus()[i]));
		}
		NotifyObservers(*this);
	}
	catch (std::exception &e) {
		MessageBox(g_hWnd, widen(e.what()).c_str(), _T("JSMonitor Update Error"), MB_OK | MB_SYSTEMMODAL);
	}
}

