#pragma once
#include "observer.h"
#include "SSDSmart.h"
#include <boost/shared_ptr.hpp>
#include <boost/tuple/tuple.hpp>

class CJSStatus {
public:
	//JSMonitor.cpp
	virtual void AddStatusDetailString(std::ostream &os, bool bHideSerial) const =0;
	virtual void UpdateSSDDialog(std::vector<HWND>& rvSmartDialogs, bool bHideSerial) const=0;
	//JSLogger.cpp
	virtual bool IsUpdated(const CJSStatus* rStatus) const=0;
	virtual std::string WriteToCSV() const=0;
	virtual bool ReadFromCSV(const std::string &rsLine)=0;
	virtual std::string MakeCSVHeader() const=0;
	//JSAlert.cpp
	virtual std::string GetStatusChangeString(const CJSStatus* pOldStatus, const std::vector<std::string> &vTargetStrings) const=0;
	virtual int GetEraseCountMax() const=0;

	virtual const CSSDStatus *GetStatus() const=0;
	virtual boost::shared_ptr<CJSStatus> MakeClone() const=0;

	static boost::tuple<int, int, double> GetEraseCountSpec() {
		return boost::make_tuple(_nSLCEraseCountMax, _nMLCEraseCountMax, _dIDXMultiply);}
	static void SetEraseCountSpec(int nSLCMax, int nMLCMax, double dIDXMultiply) {
		_nSLCEraseCountMax = nSLCMax;
		_nMLCEraseCountMax = nMLCMax;
		_dIDXMultiply = dIDXMultiply;
	}
protected:

	static int _nSLCEraseCountMax;
	static int _nMLCEraseCountMax;
	static double _dIDXMultiply;

};

class CJSSmart :
	public CSubject<CJSSmart>
{
public:
	static CJSSmart &GetInstance();
	virtual ~CJSSmart(void);

	void Update();
	size_t GetSuitableDriveNum() const {
		return _vpSSDStatus.size();
	}
	const std::vector<boost::shared_ptr<CJSStatus> > &GetSSDStatus() const {return _vpSSDStatus;}

protected:

	CSSDSmart *_pSmartImpl;
	std::vector<boost::shared_ptr<CJSStatus> > _vpSSDStatus;

	CJSSmart(CSSDSmart *pSmartImpl);
	CJSSmart(const CJSSmart& r);
	CJSSmart& operator=(const CJSSmart& r);

};
