#include "StdAfx.h"
#include "RegistrySetting.h"
#include <sstream>
#include <exception>

using namespace std;

CRegistrySetting::CRegistrySetting(void) {
	_hKey = NULL;
	_nValueType = REG_NONE;
}

CRegistrySetting::~CRegistrySetting(void)
{
	if (_hKey) RegCloseKey(_hKey);
}

void MakeVectorFromString (const wchar_t *sOriginal, vector<wstring>& vStrings) {
	const wchar_t *sCurrent = sOriginal;
	while (true) {
		wstring s1(sCurrent);
		if (s1.size() == 0) {
			break;
		}
		vStrings.push_back(s1);
		sCurrent += (s1.size()+1);
	}
}

size_t MakeStringFromVector (const vector<wstring>& vStrings, wchar_t* pTarget, size_t nBufSize) {

	if (vStrings.empty()) {
		pTarget[0] = _T('\0');
		pTarget[1] = _T('\0');
		return 2;
	}
	size_t nLength = 0;
	for (size_t i=0; i<vStrings.size(); ++i) {
		wcscpy_s(pTarget+nLength, nBufSize-nLength, vStrings[i].c_str());
		nLength += (vStrings[i].size()+1);
	}
	pTarget[nLength] = _T('\0');

	return nLength+1;
}

bool IsExpandSz(const std::wstring &rS) {
	int nCnt = 0;
	for (size_t i=0; i<rS.size(); ++i) {
		if (rS[i] == _T('%')) ++nCnt;
	}
	return (nCnt >= 2);
}

bool CRegistrySetting::SetValue(ValueType value) {

	if (!_hKey) return false;

	if (value.which() == 0) { //DWORD
		DWORD d = boost::get<DWORD>(value);
		if (RegSetValueEx(_hKey, _sKey.c_str(), 0, REG_DWORD, (const BYTE*)(&d), sizeof(DWORD)) != ERROR_SUCCESS) {
			dout << "Set failed " << narrow(_sPos.c_str()) << " " << narrow(_sKey.c_str()) << endl;
			return false;
		}
	}
	else if (value.which() == 1) { //wstring
		wstring s = boost::get<wstring>(value);
		int nType;
		if (IsExpandSz(s)) nType = REG_EXPAND_SZ;
		else nType = REG_SZ;
		if (RegSetValueEx(_hKey, _sKey.c_str(), 0, nType, (const BYTE*)(s.c_str()), sizeof(wchar_t)*(s.size()+1)) != ERROR_SUCCESS) {
			dout << "Set failed " << narrow(_sPos.c_str()) << " " << narrow(_sKey.c_str()) << endl;
			return false;
		}
	}
	else if (value.which() == 2) { //vector<wstring>
		vector<wstring> v = boost::get<vector<wstring> >(value);
		wchar_t sBuf[1000];
		size_t s = MakeStringFromVector(v, sBuf, 1000);
		if (RegSetValueEx(_hKey, _sKey.c_str(), 0, REG_MULTI_SZ, (const BYTE*)(sBuf), sizeof(wchar_t)*s) != ERROR_SUCCESS) {
			dout << "Set failed " << narrow(_sPos.c_str()) << " " << narrow(_sKey.c_str()) << endl;
			return false;
		}
	}
	_valCurrent = value;
	return true;
}

bool CRegistrySetting::Open(HKEY hRoot, const std::wstring& sPos, const std::wstring& sKey) {
	_hRoot = hRoot;
	_sPos = sPos;
	_sKey = sKey;
	const DWORD nBufSize = 1000;
	wchar_t pValue[nBufSize];

//	if (RegOpenKeyEx(hRoot, sPos.c_str(), 0, KEY_ALL_ACCESS, &_hKey) != ERROR_SUCCESS) {
	if (_hKey) RegCloseKey(_hKey);
	if (RegCreateKeyEx(hRoot, sPos.c_str(), 0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS | KEY_WOW64_64KEY, NULL, &_hKey, NULL) != ERROR_SUCCESS) {
		wstringstream ss;
		ss << "RegOpen Failed! " << sPos;
		string s = narrow(ss.str().c_str());
		exception e(s.c_str());
		throw e;
	}
	if (RegQueryValueEx(_hKey, sKey.c_str(), 0, &_nValueType, (LPBYTE)pValue, (LPDWORD)&nBufSize) != ERROR_SUCCESS) {
		dout << "Not found " << _hKey << " " << narrow(sPos.c_str()) << " " << narrow(sKey.c_str()) << endl;
		return false;
	}
	if (_nValueType == REG_DWORD) {
		DWORD* pdwCurrent = (DWORD*)pValue;
		_valCurrent = *pdwCurrent;
	}
	if ((_nValueType == REG_SZ) || (_nValueType == REG_EXPAND_SZ)) {
		wchar_t *psCurrent = (wchar_t *)pValue;
		_valCurrent = psCurrent;
	}
	if (_nValueType == REG_MULTI_SZ) {
		wchar_t *psCurrent = (wchar_t *)pValue;
		vector<wstring> vStrings;
		MakeVectorFromString(pValue, vStrings);
		_valCurrent = vStrings;
	}
	return true;
}

bool CRegistrySettingWithCheckButton::Open(HKEY hRoot, const std::wstring& sPos, const std::wstring& sKey, DWORD nType,
																					 const CRegistrySetting::ValueType &valOKState, const CRegistrySetting::ValueType &valNGState, bool bNotExistState)
{
	if ((nType != REG_DWORD) && (nType != REG_SZ)) {
		wstringstream ss;
		ss << "nType Not Support! " << nType;
		string s = narrow(ss.str().c_str());
		exception e(s.c_str());
		throw e;
	}
	const DWORD nBufSize = 1000;
	_bNotExistState = bNotExistState;
	_valNGState = valNGState;
	_valOKState = valOKState;

	try {
		if (!CRegistrySetting::Open(hRoot, sPos, sKey)) {
			//̃L[݂Ȃ
			_bStatus = _bNotExistState;
			_nValueType = nType;
			dout << "Not found " << _hKey << endl;
		}
		else {
			if (_nValueType != nType) {
				wstringstream ss;
				ss << "RegType Uncompatible!" << _nValueType << " " << nType;
				string s = narrow(ss.str().c_str());
				exception e(s.c_str());
				throw e;
			}
			if (_valOKState == _valCurrent) {
				_bStatus = true;
			}
			else _bStatus = false;
			if (_bStatus) {
				dout << "OK " << narrow(sPos.c_str()) << " " << narrow(sKey.c_str()) << endl;
			}
			else {
				dout << "NG " << narrow(sPos.c_str()) << " " << narrow(sKey.c_str()) << endl;
			}
		}
	}
	catch (std::exception &e) {
		//eGg瑶݂Ȃꍇ
		dout << "CRegistrySettingWithCheckButton open parent failed: " << narrow(sPos) << " " << e.what() << endl;
		return false;
	}
	dout << "Open " << _hKey << " " << narrow(_sKey) << endl;
	return true;
}

bool CRegistrySettingWithCheckButton::SetStatus(bool bStatus) {

	if (bStatus == _bStatus) return true;
	if (_nValueType == REG_DWORD) {
		DWORD dData;
		if (bStatus) {
			dData = boost::get<DWORD>(_valOKState);
		}
		else {
			dData = boost::get<DWORD>(_valNGState);
		}
		dout << "SetStatus " << _hKey << " " << narrow(_sKey) << endl;
		if (RegSetValueEx(_hKey, _sKey.c_str(), 0, _nValueType, (const BYTE*)(&dData), sizeof(DWORD)) != ERROR_SUCCESS) {
			dout << "reg setting failed!" << endl;
			return false;
		}
		_bStatus = bStatus;
	}
	else if (_nValueType == REG_SZ) {
		wstring sData;
		if (bStatus) {
			sData = boost::get<wstring>(_valOKState);
		}
		else {
			sData = boost::get<wstring>(_valNGState);
		}
		if (RegSetValueEx(_hKey, _sKey.c_str(), 0, _nValueType, (const BYTE*)(sData.c_str()), sizeof(wchar_t)*(sData.size()+1)) != ERROR_SUCCESS) {
			dout << "reg setting failed2!" << endl;
			return false;
		}
		_bStatus = bStatus;
	}
	else {
		dout << "unknown error!" << endl;
	}
	return true;
}

bool CRegistrySetting::RemoveValue() {
	
	if (_hKey) {
		return (RegDeleteValue(_hKey, _sKey.c_str()) == ERROR_SUCCESS);
	}
	return true;
}
