#include "StdAfx.h"
#include "SettingForSSD.h"
#include "OutputDebugStream.h"
#include <windowsx.h> 
#include <sstream>
#include <exception>
#include "resource.h"
#include <shlobj.h>
#include "DiskState.h"
#include <fstream>
#include <boost/date_time/posix_time/posix_time.hpp>

#pragma comment(lib, "comctl32.lib")

using namespace std;

CDriveSetting::CDriveSetting(const std::wstring& sDriveLetter){
	_sDriveLetter = sDriveLetter;
	_nPagingFileStatus = eNoPagingFile;
	_nPageSizeFirst = 0;
	_nPageSizeMax   = 0;
}
bool CDriveSetting::SetPagingFilesSetting(const std::wstring& sStr) {
	std::wstringstream wss(sStr);
	std::wstring tmp;
	wss >> tmp >> _nPageSizeFirst >> _nPageSizeMax;
	if (tmp[0] != _sDriveLetter[0]) return false;

	if ((_nPageSizeFirst == 0) && (_nPageSizeMax == 0)) _nPagingFileStatus = eSystem;
	else _nPagingFileStatus = eCustom;
	return true;
}

bool CDriveSetting::SetPagingFilesSetting(EPagingSizeType eType, unsigned int nFirst, unsigned int nMax) {
	_nPagingFileStatus = eType;
	_nPageSizeFirst = nFirst;
	_nPageSizeMax = nMax;
	return true;
}

bool CDiskSetting::OpenDrive(int nDriveNumber, const std::wstring &rsDriveName) {
	_nDriveNumber = nDriveNumber;
	std::wstringstream ss;
	ss << rsDriveName << _T(" (Drive #") << nDriveNumber << _T(")");
	_sDriveName = ss.str();

	DWORD	dwWritten=0;
	BOOL	ret;
	HANDLE	hDevice;
	TCHAR	pszDevice[256];
	::_stprintf_s (pszDevice,256,_T("\\\\.\\PhysicalDrive%d"),nDriveNumber);
	hDevice = ::CreateFile(pszDevice,GENERIC_READ | GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
	if(hDevice == INVALID_HANDLE_VALUE) {
		dout << "fail1" << std::endl;
		return	false;
	}

	DISK_CACHE_INFORMATION	sInfo;
	::ZeroMemory(&sInfo,sizeof(DISK_CACHE_INFORMATION));
	ret = ::DeviceIoControl(hDevice,IOCTL_DISK_GET_CACHE_INFORMATION,NULL,0,&sInfo,sizeof(DISK_CACHE_INFORMATION),&dwWritten,NULL);
	::CloseHandle(hDevice);
//	dout << "opendrive " << ret << " " << dwWritten << std::endl;

	if(ret == FALSE || dwWritten != sizeof(DISK_CACHE_INFORMATION)) {
		dout << "fail2 " << ret << " " << dwWritten << std::endl;
		return false;
	}
	_bWriteCache = (sInfo.WriteCacheEnabled != FALSE);
	_bReadCache = (sInfo.ReadCacheEnabled != FALSE);
	return	true;
}

void CDiskSetting::SetCacheStatusSetting(bool bWrite, bool bRead)  {
	_bWriteCache = bWrite;
	_bReadCache = bRead;
}

int CDiskSetting::ApplySetting() {
	DWORD	dwWritten;
	BOOL	ret;
	HANDLE	hDevice;
	TCHAR	pszDevice[256];

	::_stprintf_s (pszDevice,256,_T("\\\\.\\PhysicalDrive%d"),_nDriveNumber);
	hDevice = ::CreateFile(pszDevice,GENERIC_READ | GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,NULL,OPEN_EXISTING,0,NULL);
	if(hDevice == INVALID_HANDLE_VALUE) {
//		dout << "fail1" << endl;
		return 0;
	}

	DISK_CACHE_INFORMATION	sInfo;
	DISK_CACHE_INFORMATION	sInfo2;
	::ZeroMemory(&sInfo,sizeof(DISK_CACHE_INFORMATION));
	::ZeroMemory(&sInfo2,sizeof(DISK_CACHE_INFORMATION));
	ret = ::DeviceIoControl(hDevice,IOCTL_DISK_GET_CACHE_INFORMATION,NULL,0,&sInfo,sizeof(DISK_CACHE_INFORMATION),&dwWritten,NULL);
	sInfo2 = sInfo;
	if ( (((BOOLEAN)_bWriteCache) == sInfo.WriteCacheEnabled) && (((BOOLEAN)_bReadCache) == sInfo.ReadCacheEnabled) ) {
		return 1;
	}

	sInfo.WriteCacheEnabled = _bWriteCache;
	sInfo.ReadCacheEnabled  = _bReadCache;

	ret = ::DeviceIoControl(hDevice,IOCTL_DISK_SET_CACHE_INFORMATION,&sInfo,sizeof(DISK_CACHE_INFORMATION),&sInfo2,sizeof(DISK_CACHE_INFORMATION),&dwWritten,NULL);
	::CloseHandle(hDevice);
//	if(ret == FALSE || dwWritten != sizeof(DISK_CACHE_INFORMATION)) {
	if(ret == FALSE) {
//		dout << "fail2 " << ret << " " << dwWritten << endl;
		return	0;
	}
	return 2;
}

CSettingForSSD::CSettingForSSD() {
	_hDlg = NULL;
	_nCurrentSelectedDrive = 0;
	_nCurrentSelectedDisk = 0;
};
CSettingForSSD::~CSettingForSSD(void) {

	for (size_t i=0; i<_vServices.size(); ++i) {
		CloseServiceHandle(_vServices[i].get<0>());
	}
};

bool CSettingForSSD::AddRegistrySetting(int nDlgItem, HKEY hRoot, const std::wstring& sPos, const std::wstring& sKey, DWORD nType, 
																				const boost::variant<DWORD,std::wstring> &valOKState, const boost::variant<DWORD,std::wstring> &valNGState, bool bNotExistState) 
{
	boost::shared_ptr<CRegistrySettingWithCheckButton> prs(new CRegistrySettingWithCheckButton);
	if (!prs->Open(hRoot, sPos, sKey, nType, valOKState, valNGState, bNotExistState)) {
		return false;
	}

	HWND hHandle = GetDlgItem(_hDlg, nDlgItem);
	if (prs->GetStatus()) Button_SetCheck(hHandle, BST_CHECKED);
	else Button_SetCheck(GetDlgItem(_hDlg, nDlgItem), BST_UNCHECKED);

	_vRegistrySettingWithCheckButton.push_back(make_pair(hHandle, prs));
	return true;
}

extern HINSTANCE g_hInst;

void CSettingForSSD::AddToolTips() {

	InitCommonControls();
	HWND hTool = CreateWindowEx( 0 , TOOLTIPS_CLASS ,
		NULL , TTS_ALWAYSTIP | TTS_BALLOON ,
		CW_USEDEFAULT , CW_USEDEFAULT ,
		CW_USEDEFAULT , CW_USEDEFAULT ,
		_hDlg , NULL , g_hInst ,
		NULL
	);
	SendMessage(hTool, TTM_SETMAXTIPWIDTH, 0, (LPARAM)400);

	wchar_t sInfo[1000];
	vector<pair<HWND, wstring> > vToolTipInfo;
	TOOLINFO ti;

	//Ȃ2ʂ蕪ȂƂȂ񂾂낤c
	LoadString(g_hInst, IDS_INDEXING, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_COMBO_INDEXSERVICE_STATE), wstring(sInfo)));
	LoadString(g_hInst, IDS_PREFETCH, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_PREFETCH), wstring(sInfo)));
	LoadString(g_hInst, IDS_83NAME, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_83), wstring(sInfo)));
	LoadString(g_hInst, IDS_FILEACCESS, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_FILEACCESSTIME), wstring(sInfo)));
	LoadString(g_hInst, IDS_AUTODEFRAG, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_AUTODEFRAG), wstring(sInfo)));
	LoadString(g_hInst, IDS_BOOTOPTIMIZE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_BOOTOPTIMIZATION), wstring(sInfo)));
	LoadString(g_hInst, IDS_LARGESYSTEMCACHE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_LARGESYSTEMCACHE), wstring(sInfo)));
	LoadString(g_hInst, IDS_NTFSMEMORYUSAGE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_PAGEDPOOLMEMORY), wstring(sInfo)));
	LoadString(g_hInst, IDS_SUPERFETCH_REG, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_SUPERFETCH), wstring(sInfo)));
	LoadString(g_hInst, IDS_SUPERFETCH_SERVICE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_COMBO_SUPERFETCH_STATE), wstring(sInfo)));
	LoadString(g_hInst, IDS_WINDOWSSEARCH, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_COMBO_WINDOWSSEARCH_STATE), wstring(sInfo)));
	LoadString(g_hInst, IDS_TEMPFOLDER, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_BUTTON_TEMPUSER), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_BUTTON_TMPUSER), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_BUTTON_TEMPSYSTEM), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_BUTTON_TMPSYSTEM), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_BUTTON_CHANGEALLTEMP), wstring(sInfo)));
	LoadString(g_hInst, IDS_DISKSETTING, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_COMBO_DISK), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_WRITECACHE), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_CHECK_READCACHE), wstring(sInfo)));
	LoadString(g_hInst, IDS_PAGINGFILE, sInfo ,1000);
//	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_BUTTON_DISABLEPAGING), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_COMBO_DRIVE), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_RADIO_PAGINGNO), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_RADIO_PAGINGSYSTEM), wstring(sInfo)));
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_RADIO_PAGINGCUSTOM), wstring(sInfo)));

	for (size_t i=0; i<vToolTipInfo.size(); ++i) {

		ZeroMemory(&ti, sizeof(TOOLINFO));
		GetWindowRect(vToolTipInfo[i].first, &ti.rect);
    ti.cbSize = sizeof (TOOLINFO);
    ti.uFlags = TTF_IDISHWND | TTF_SUBCLASS;
    ti.hwnd = _hDlg;
		ti.uId = (UINT)vToolTipInfo[i].first;
    ti.hinst = g_hInst;
		ti.lpszText = (LPWSTR)vToolTipInfo[i].second.c_str();
		SendMessage(hTool , TTM_ADDTOOL , 0 , (LPARAM)&ti);
	}

	vToolTipInfo.clear();
	LoadString(g_hInst, IDS_TEMPFOLDER, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_TEMPFOLDERGROUP), wstring(sInfo)));
	LoadString(g_hInst, IDS_DISKSETTING, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_DISKGROUP), wstring(sInfo)));
	LoadString(g_hInst, IDS_PAGINGFILE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_PAGEGROUP), wstring(sInfo)));
	LoadString(g_hInst, IDS_ABOUTSERVICE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_SERVICEGROUP), wstring(sInfo)));
	LoadString(g_hInst, IDS_INDEXING, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_INDEXING_S), wstring(sInfo)));
	LoadString(g_hInst, IDS_WINDOWSSEARCH, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_WINDOWSSEARCH_S), wstring(sInfo)));
	LoadString(g_hInst, IDS_SUPERFETCH_SERVICE, sInfo ,1000);
	vToolTipInfo.push_back(make_pair(GetDlgItem(_hDlg, IDC_SUPERFETCH_S), wstring(sInfo)));

	for (size_t i=0; i<vToolTipInfo.size(); ++i) {

		ZeroMemory(&ti, sizeof(TOOLINFO));
		GetWindowRect(vToolTipInfo[i].first, &ti.rect);
		POINT p1={ti.rect.left, ti.rect.top};
		POINT p2={ti.rect.right,ti.rect.bottom};
		ScreenToClient(_hDlg, &p1);
		ScreenToClient(_hDlg, &p2);
		ti.rect.left = p1.x;
		ti.rect.right = p2.x;
		ti.rect.top = p1.y;
		ti.rect.bottom = p2.y;
		ti.cbSize = sizeof (TOOLINFO);
		ti.uFlags = TTF_SUBCLASS | TTF_TRANSPARENT;
		ti.hwnd = _hDlg;
		ti.hinst = g_hInst;
		ti.lpszText = (LPWSTR)vToolTipInfo[i].second.c_str();
		SendMessage(hTool , TTM_ADDTOOL , 0 , (LPARAM)&ti);
	}

	SendMessage(hTool, TTM_SETDELAYTIME, (WPARAM)TTDT_AUTOPOP, 1200*1000);
}

wstring ChooseDialog(HWND hDlg) {
	BROWSEINFO bi;
	wchar_t aResult[_MAX_PATH];
	bi.hwndOwner = hDlg;
	bi.pidlRoot = 0;
	bi.pszDisplayName = aResult;
	bi.lpszTitle = _T("tH_Iĉ");
	bi.ulFlags = 0;
	bi.lpfn = 0;
	bi.lParam = 0;
	// tH_I_CAO̕\ 
	LPITEMIDLIST pIDList = SHBrowseForFolder(&bi);
	if ( (pIDList != NULL) && SHGetPathFromIDList(pIDList, aResult)) {
		CoTaskMemFree(pIDList);
		return aResult;
	}
	else{
		if (pIDList) CoTaskMemFree(pIDList);
		dout << "failed open" << endl;
		return _T("");
	}
}

bool CSettingForSSD::ChangeTempFolder(unsigned int nID) {

	wstring sFolder = ChooseDialog(_hDlg);
	if (sFolder != _T("")) {
		unsigned int nID2 = 0;
		if (nID == IDC_BUTTON_TEMPSYSTEM) {
			_sSystemTEMPDir = sFolder;
			nID2 = IDC_TEMP_SYSTEM_DIR;
		}
		else if (nID == IDC_BUTTON_TMPSYSTEM) {
			_sSystemTMPDir = sFolder;
			nID2 = IDC_TMP_SYSTEM_DIR;
		}
		else if (nID == IDC_BUTTON_TEMPUSER) {
			_sUserTEMPDir = sFolder;
			nID2 = IDC_TEMP_USER_DIR;
		}
		else if (nID == IDC_BUTTON_TMPUSER) {
			_sUserTMPDir = sFolder;
			nID2 = IDC_TMP_USER_DIR;
		}
		
		if (nID == IDC_BUTTON_CHANGEALLTEMP) {
			_sSystemTMPDir = sFolder;
			_sUserTMPDir = sFolder;
			_sSystemTEMPDir = sFolder;
			_sUserTEMPDir = sFolder;
			SetDlgItemText(_hDlg, IDC_TEMP_USER_DIR, sFolder.c_str());
			SetDlgItemText(_hDlg, IDC_TMP_USER_DIR, sFolder.c_str());
			SetDlgItemText(_hDlg, IDC_TEMP_SYSTEM_DIR, sFolder.c_str());
			SetDlgItemText(_hDlg, IDC_TMP_SYSTEM_DIR, sFolder.c_str());
		}
		else if (nID2 != 0) {
			SetDlgItemText(_hDlg, nID2, sFolder.c_str());
		}
		else {
			dout << "ChangeTempFolder Unknown Error :" << nID << endl;
			return false;
		}		
		return true;
	}
	else {
		return false;
	}
}

void CSettingForSSD::DisableAllPagingFile() {
	for (size_t i=0; i<_vDriveSetting.size(); ++i) {
		_vDriveSetting[i].SetPagingFilesSetting(CDriveSetting::eNoPagingFile,0,0);
	}
}

INT_PTR CALLBACK CSettingForSSD::Proc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {

	HWND hEdit1  = GetDlgItem(_hDlg, IDC_EDIT_PAGINGFIRST);
	HWND hEdit2  = GetDlgItem(_hDlg, IDC_EDIT_PAGINGMAX);
	HWND hRadio1 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGNO);
	HWND hRadio2 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGSYSTEM);
	HWND hRadio3 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGCUSTOM);
	HWND hComboDrive = GetDlgItem(_hDlg, IDC_COMBO_DRIVE);
	HWND hComboDisk = GetDlgItem(_hDlg, IDC_COMBO_DISK);
	switch (message)
	{
	case WM_INITDIALOG:
		if (!Init(hDlg)) {
			DestroyWindow(hDlg);
			return (INT_PTR)FALSE;
		}
		SetWindowPos(hDlg, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK){
			if (ApplySetting()) {
				EndDialog(hDlg, LOWORD(wParam));
			}
			return (INT_PTR)TRUE;			
		}
		else if (LOWORD(wParam) == IDCANCEL)
		{
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		else if (LOWORD(wParam) == IDAPPLY)
		{
			ApplySetting();
			return (INT_PTR)TRUE;
		}
		else if ( (LOWORD(wParam) == IDC_BUTTON_TEMPUSER) || (LOWORD(wParam) == IDC_BUTTON_TMPUSER) || 
							(LOWORD(wParam) == IDC_BUTTON_TEMPSYSTEM) || (LOWORD(wParam) == IDC_BUTTON_TMPSYSTEM) || 
							(LOWORD(wParam) == IDC_BUTTON_CHANGEALLTEMP) )
		{
			ChangeTempFolder(LOWORD(wParam));
		}
		else if (LOWORD(wParam) == IDC_BUTTON_DISABLEPAGING) {
			DisableAllPagingFile();
			EnableWindow(hEdit1, false);
			EnableWindow(hEdit2, false);
			SetDlgItemInt(hDlg, IDC_EDIT_PAGINGFIRST, 0, true);
			SetDlgItemInt(hDlg, IDC_EDIT_PAGINGMAX,   0, true);
			SendMessage(hRadio1, BM_CLICK, 0, 0);
		}
		else if ( (LOWORD(wParam) == IDC_RADIO_PAGINGNO) && (BST_CHECKED == SendMessage(GetDlgItem(hDlg, IDC_RADIO_PAGINGNO),BM_GETCHECK,0,0)) ){
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGFIRST), false);
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGMAX), false);
		}
		else if ( (LOWORD(wParam) == IDC_RADIO_PAGINGSYSTEM) && (BST_CHECKED == SendMessage(GetDlgItem(hDlg, IDC_RADIO_PAGINGSYSTEM),BM_GETCHECK,0,0)) ){
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGFIRST), false);
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGMAX), false);
		}
		else if ( (LOWORD(wParam) == IDC_RADIO_PAGINGCUSTOM) && (BST_CHECKED == SendMessage(GetDlgItem(hDlg, IDC_RADIO_PAGINGCUSTOM),BM_GETCHECK,0,0)) ){
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGFIRST), true);
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGMAX), true);
		}
		else if (LOWORD(wParam) == IDC_CHECK_AUTOPAGING) {
			bool bState = (Button_GetCheck(GetDlgItem(hDlg, IDC_CHECK_AUTOPAGING)) != BST_CHECKED);
			EnableWindow(GetDlgItem(hDlg, IDC_COMBO_DRIVE), bState);
			EnableWindow(GetDlgItem(hDlg, IDC_RADIO_PAGINGCUSTOM), bState);
			EnableWindow(GetDlgItem(hDlg, IDC_RADIO_PAGINGNO), bState);
			EnableWindow(GetDlgItem(hDlg, IDC_RADIO_PAGINGSYSTEM), bState);
			EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_DISABLEPAGING), bState);
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGFIRST), bState);
			EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGMAX), bState);
			SetDriveStatusToGUI();
		}
		else if ((LOWORD(wParam) == IDC_COMBO_DRIVE) && (HIWORD(wParam) == CBN_SELCHANGE)) {
			ReadDriveStatusFromGUI();
			_nCurrentSelectedDrive = SendMessage(hComboDrive, CB_GETCURSEL, 0, 0);
			SetDriveStatusToGUI();
		}
		else if ((LOWORD(wParam) == IDC_COMBO_DISK) && (HIWORD(wParam) == CBN_SELCHANGE)) {
			//ύX̏Ԃݒ
			ReadDiskStatusFromGUI();
			_nCurrentSelectedDisk = SendMessage(hComboDisk, CB_GETCURSEL, 0, 0);
			SetDiskStatusToGUI();
		}

		break;
	}
	return (INT_PTR)FALSE;
}

void CSettingForSSD::ReadDiskStatusFromGUI() {

	if (_vDiskSetting.empty()) return;
	HWND hcheckWriteCache = GetDlgItem(_hDlg, IDC_CHECK_WRITECACHE);
	HWND hcheckReadCache  = GetDlgItem(_hDlg, IDC_CHECK_READCACHE);
	HWND hCombo = GetDlgItem(_hDlg, IDC_COMBO_DISK);
	_vDiskSetting[_nCurrentSelectedDisk].SetCacheStatusSetting(
		(Button_GetCheck(hcheckWriteCache) != FALSE), (Button_GetCheck(hcheckReadCache) != FALSE) );
}

void CSettingForSSD::SetDiskStatusToGUI() {

	if (_vDiskSetting.empty()) return;
	HWND hcheckWriteCache = GetDlgItem(_hDlg, IDC_CHECK_WRITECACHE);
	HWND hcheckReadCache  = GetDlgItem(_hDlg, IDC_CHECK_READCACHE);
	bool bWrite, bRead;
	boost::tie(bWrite, bRead) = _vDiskSetting[_nCurrentSelectedDisk].GetChacheStatus();
	Button_SetCheck(hcheckWriteCache, bWrite);
	Button_SetCheck(hcheckReadCache,  bRead);
}


bool CSettingForSSD::ReadDriveStatusFromGUI() {

	HWND hEdit1  = GetDlgItem(_hDlg, IDC_EDIT_PAGINGFIRST);
	HWND hEdit2  = GetDlgItem(_hDlg, IDC_EDIT_PAGINGMAX);
	HWND hRadio1 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGNO);
	HWND hRadio2 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGSYSTEM);
	HWND hRadio3 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGCUSTOM);
	HWND hCombo = GetDlgItem(_hDlg, IDC_COMBO_DRIVE);

	//݂̏Ԃ擾
	if (Button_GetCheck(hRadio1)) {
		_vDriveSetting[_nCurrentSelectedDrive].SetPagingFilesSetting(CDriveSetting::eNoPagingFile,0,0);
	}
	else if (Button_GetCheck(hRadio2)) {
		_vDriveSetting[_nCurrentSelectedDrive].SetPagingFilesSetting(CDriveSetting::eSystem,0,0);
	}
	else if (Button_GetCheck(hRadio3)) {
		BOOL bSuccess1, bSuccess2;
		int nFirst = GetDlgItemInt(_hDlg, IDC_EDIT_PAGINGFIRST, &bSuccess1, TRUE);
		int nMax = GetDlgItemInt(_hDlg, IDC_EDIT_PAGINGMAX, &bSuccess2, TRUE);
		dout << nFirst << " " << nMax << endl;
		if (bSuccess1 && bSuccess2 && (nFirst > 0) && (nMax > 0)) {
			if (nFirst <= nMax) {
				_vDriveSetting[_nCurrentSelectedDrive].SetPagingFilesSetting(CDriveSetting::eCustom, nFirst, nMax);
			}
			else {
				MessageBox(_hDlg, _T("őlɂ͏lȏ̒lݒ肵Ă"), _T("Error"), MB_OK);
				return false;
			}
		}
		else {
			MessageBox(_hDlg, _T("y[WOt@C̃TCY1ȏ̐Ŏw肵Ă"), _T("Error"), MB_OK);
			return false;
		}
	}
	return true;
}


void CSettingForSSD::SetDriveStatusToGUI() {

	HWND hEdit1  = GetDlgItem(_hDlg, IDC_EDIT_PAGINGFIRST);
	HWND hEdit2  = GetDlgItem(_hDlg, IDC_EDIT_PAGINGMAX);
	HWND hRadio1 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGNO);
	HWND hRadio2 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGSYSTEM);
	HWND hRadio3 = GetDlgItem(_hDlg, IDC_RADIO_PAGINGCUSTOM);

	CDriveSetting::EPagingSizeType n1;
	unsigned int n2, n3;
	boost::tie(n1,n2,n3) = _vDriveSetting[_nCurrentSelectedDrive].GetPagingFileStatus();
	if (n1==CDriveSetting::eNoPagingFile) {
		SendMessage(hRadio1, BM_CLICK, 0, 0);
		EnableWindow(hEdit1, false);
		EnableWindow(hEdit2, false);
		SetDlgItemInt(_hDlg, IDC_EDIT_PAGINGFIRST, 0, true);
		SetDlgItemInt(_hDlg, IDC_EDIT_PAGINGMAX,   0, true);
	}
	else if (n1==CDriveSetting::eSystem) {
		SendMessage(hRadio2, BM_CLICK, 0, 0);
		EnableWindow(hEdit1, false);
		EnableWindow(hEdit2, false);
		SetDlgItemInt(_hDlg, IDC_EDIT_PAGINGFIRST, 0, true);
		SetDlgItemInt(_hDlg, IDC_EDIT_PAGINGMAX,   0, true);
	}
	else {
		SendMessage(hRadio3, BM_CLICK, 0, 0);
		EnableWindow(hEdit1, true);
		EnableWindow(hEdit2, true);
		SetDlgItemInt(_hDlg, IDC_EDIT_PAGINGFIRST, n2, true);
		SetDlgItemInt(_hDlg, IDC_EDIT_PAGINGMAX,   n3, true);
	}
}

static boost::shared_ptr<CSettingForSSD> pInstance;
void InitSSDSetting() {
	if(!pInstance) pInstance.reset();
	pInstance = boost::shared_ptr<CSettingForSSD>(new CSettingForSSD);
}

INT_PTR CALLBACK SSDSettingProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	return pInstance->Proc(hDlg, message, wParam, lParam);
}

#include <boost/assign.hpp>

void CSettingForSSD::OpenServices(ofstream &ofs) {

	SC_HANDLE hSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if (hSCManager == NULL) {
		throw (exception("OpenSCManager Failed!"));
	}
	for (size_t i=0; i<_vServices.size(); ++i) {
		if (_vServices[i].get<0>()) CloseServiceHandle(_vServices[i].get<0>());
	}
	_vServices.clear();

	vector<pair<std::string, int> > vServiceCandidates;
	vServiceCandidates.push_back(make_pair( "Indexing Service", IDC_COMBO_INDEXSERVICE_STATE));
	vServiceCandidates.push_back(make_pair( "Windows Search", IDC_COMBO_WINDOWSSEARCH_STATE));
	vServiceCandidates.push_back(make_pair( "Superfetch", IDC_COMBO_SUPERFETCH_STATE));

	for (size_t i=0; i<vServiceCandidates.size(); ++i) {
		string sServiceName = vServiceCandidates[i].first;
		HWND hCombo = GetDlgItem(_hDlg, vServiceCandidates[i].second);
		char szServiceName[256];
		DWORD dwLength = sizeof(szServiceName);
		if (!GetServiceKeyNameA(hSCManager, sServiceName.c_str(), szServiceName, &dwLength)) { //ȂW͎sH
			ofs << sServiceName << "/" << szServiceName <<  " ServiceKeyName Failed!" << endl;
			EnableWindow(hCombo, false);
		}
		else {
			SC_HANDLE hService = OpenServiceA(hSCManager, szServiceName, SERVICE_CHANGE_CONFIG|SERVICE_QUERY_CONFIG);
			if (hService == NULL) {
				CloseServiceHandle(hSCManager);
				EnableWindow(hCombo, false);
				ofs << sServiceName << "/" << szServiceName <<  " OpenService Failed!" << endl;
			}
			else {
				DWORD dwSize;
				BOOL b1 = QueryServiceConfig(hService, NULL, 0, &dwSize);
				QUERY_SERVICE_CONFIG *lpServiceConfig = (LPQUERY_SERVICE_CONFIG)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwSize);
				BOOL b2 = QueryServiceConfig(hService, lpServiceConfig, dwSize, &dwSize);
				if (b2) {
					ofs << sServiceName << "/" << szServiceName << " OK" << endl;
				}
				else {
					ostringstream oss;
					oss << sServiceName << "QueryService Failed";
					throw exception(oss.str().c_str());
				}
				SendMessage(hCombo, CB_INSERTSTRING, 0, (LPARAM)_T(""));
				SendMessage(hCombo, CB_INSERTSTRING, 1, (LPARAM)_T("蓮"));
				SendMessage(hCombo, CB_INSERTSTRING, 2, (LPARAM)_T(""));

				if (lpServiceConfig->dwStartType == SERVICE_AUTO_START) SendMessage(hCombo, CB_SETCURSEL, 0, 0);
				else if (lpServiceConfig->dwStartType == SERVICE_DEMAND_START) SendMessage(hCombo, CB_SETCURSEL, 1, 0);
				else SendMessage(hCombo, CB_SETCURSEL, 2, 0);
				HeapFree(GetProcessHeap(), 0, lpServiceConfig);
				_vServices.push_back(boost::make_tuple<SC_HANDLE, std::wstring, HWND>(hService, widen(sServiceName), hCombo));
			}
		}
	}
	CloseServiceHandle(hSCManager);
}

using namespace boost::assign;

bool CSettingForSSD::Init(HWND hDlg) {

	_hDlg = hDlg;
	ofstream ofs("JSOptimizer_log.txt", std::ios::out | std::ios::app);
	try {
		if (!ofs.is_open()) throw (exception("Log File Open Failed!"));
		ofs << endl << endl << "SettingForSSD Init: " << boost::posix_time::second_clock::local_time() << endl;

		//Registry Tweak Init
		_vRegistrySettingWithCheckButton.clear();
		vector<int> anRegControls;
		vector<wstring> asRegPos;
		vector<wstring> asRegKey;
		vector<int> anRegEnable;
		vector<int> anRegDisable;
		vector<bool> abRegNotExist;
		anRegControls += IDC_CHECK_PREFETCH, IDC_CHECK_83, IDC_CHECK_FILEACCESSTIME,
										 IDC_CHECK_PAGEDPOOLMEMORY, IDC_CHECK_AUTODEFRAG;
		asRegPos += _T("SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Memory Management\\PrefetchParameters"),
								_T("SYSTEM\\CurrentControlSet\\Control\\FileSystem"),
								_T("SYSTEM\\CurrentControlSet\\Control\\FileSystem"),
								_T("SYSTEM\\CurrentControlSet\\Control\\FileSystem"), 
								_T("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\OptimalLayout"); 
		asRegKey += _T("Enableprefetcher"), _T("NtfsDisable8dot3nameCreation"), 
								_T("NtfsDisableLastAccessUpdate"), _T("NtfsMemoryUsage"), _T("EnableAutoLayout"); 
		anRegEnable += 0, 1, 1, 2, 0;
		anRegDisable += 3, 0, 0, 1, 1;
		abRegNotExist += true, false, false, false, false;

		if (IsOSWinXP()) {
			anRegControls.push_back(IDC_CHECK_LARGESYSTEMCACHE);
			asRegPos.push_back(_T("SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Memory Management"));
			asRegKey.push_back(_T("LargeSystemCache"));
			anRegEnable += 1;
			anRegDisable += 0;
			abRegNotExist += false;
			EnableWindow(GetDlgItem(hDlg, IDC_CHECK_SUPERFETCH), false);
		}
		if (IsOSVista()) {
			anRegControls.push_back(IDC_CHECK_SUPERFETCH);
			asRegPos.push_back(_T("SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Memory Management\\PrefetchParameters"));
			asRegKey.push_back(_T("EnableSuperfetch"));
			anRegEnable += 0;
			anRegDisable += 3;
			abRegNotExist += true;
			EnableWindow(GetDlgItem(hDlg, IDC_CHECK_LARGESYSTEMCACHE), false);
		}

		ofs << "Registry Setting Setup " ;
		for (size_t i=0; i<anRegControls.size(); ++i) {
			if (!AddRegistrySetting(anRegControls[i], HKEY_LOCAL_MACHINE, asRegPos[i], asRegKey[i],
															REG_DWORD, anRegEnable[i], anRegDisable[i], abRegNotExist[i]))
			{
				EnableWindow(GetDlgItem(_hDlg, anRegControls[i]), false);
				ofs << " add failed ";
			}
			ofs << i << " ";
		}
		if (!AddRegistrySetting(IDC_CHECK_BOOTOPTIMIZATION, HKEY_LOCAL_MACHINE, _T("SOFTWARE\\Microsoft\\Dfrg\\BootOptimizeFunction"), _T("Enable"), 
														REG_SZ, _T("N"), _T("Y"), false))
		{
			EnableWindow(GetDlgItem(_hDlg, IDC_CHECK_BOOTOPTIMIZATION), false);
			ofs << " add failed ";
//			throw (exception("RegSetting Failed!"));
		}
		ofs << "OK" << endl;

		//Temp Folders Init
		_vTEMPFolders.clear();
		int anTempControls[] = {IDC_TEMP_SYSTEM_DIR, IDC_TMP_SYSTEM_DIR, IDC_TEMP_USER_DIR, IDC_TMP_USER_DIR};
		HKEY ahTempRoots[]   = {HKEY_LOCAL_MACHINE,  HKEY_LOCAL_MACHINE, HKEY_CURRENT_USER, HKEY_CURRENT_USER};
		wstring asTempPos[]  = {_T("SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment"),
														_T("SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment"),
														_T("Environment"),_T("Environment")};
		wstring asTempKey[]  = {_T("TEMP"), _T("TMP"), _T("TEMP"), _T("TMP")};
		ofs << "Temp Folders Setup " ;
		for (size_t i=0; i<4; ++i) {
			boost::shared_ptr<CRegistrySetting> pReg(new CRegistrySetting());
			if (pReg->Open(ahTempRoots[i], asTempPos[i], asTempKey[i])) {
				SetDlgItemText(_hDlg, anTempControls[i], boost::get<wstring>(pReg->GetCurrentValue()).c_str());
				_vTEMPFolders.push_back(make_pair(anTempControls[i], pReg));
				ofs << i << " ";
			}
			else {
				_vTEMPFolders.push_back(make_pair(anTempControls[i], pReg));
				ofs << i << " ";
//				throw (exception("Temp Folders Init Failed!"));
			}
		}
		ofs << "OK" << endl;

		//drive init
    int flag=1;
		ofs << "Drive Setup ";
    DWORD drive=GetLogicalDrives();
		_vDriveSetting.clear();
		wchar_t sDriveLetter[4];
    for(int n=0;n<25;n++,flag<<=1) {
      if(drive&flag) {
        wsprintf(sDriveLetter,_T("%c:\\"),'A'+n);
				if (GetDriveType(sDriveLetter) == DRIVE_FIXED) {
					CDriveSetting s1(sDriveLetter);
					_vDriveSetting.push_back(s1);
					ofs << narrow(sDriveLetter) << " ";
				}
			}
    }
		ofs << "OK" << endl;

		//pagefile
		ofs << "Pagefile Setup ";
		if ( !_PageFileSetting.Open(HKEY_LOCAL_MACHINE, _T("SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Memory Management"), _T("PagingFiles"))
			|| (_PageFileSetting.GetValueType() != REG_MULTI_SZ))
		{
			throw (exception("Paging file setting not found!"));
		}

		if (IsOSWinXP()) EnableWindow(GetDlgItem(hDlg, IDC_CHECK_AUTOPAGING), false);

		vector<wstring> vsPageFiles = boost::get<vector<wstring> >(_PageFileSetting.GetCurrentValue());
		for (size_t i=0; i<vsPageFiles.size(); ++i) {
			if (vsPageFiles[i][0] == _T('?')) {
				Button_SetCheck(GetDlgItem(hDlg, IDC_CHECK_AUTOPAGING), BST_CHECKED);
				EnableWindow(GetDlgItem(hDlg, IDC_COMBO_DRIVE), false);
				EnableWindow(GetDlgItem(hDlg, IDC_RADIO_PAGINGCUSTOM), false);
				EnableWindow(GetDlgItem(hDlg, IDC_RADIO_PAGINGNO), false);
				EnableWindow(GetDlgItem(hDlg, IDC_RADIO_PAGINGSYSTEM), false);
				EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_DISABLEPAGING), false);
				EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGFIRST), false);
				EnableWindow(GetDlgItem(hDlg, IDC_EDIT_PAGINGMAX), false);
				for (size_t j=0; j<_vDriveSetting.size(); ++j) {
					_vDriveSetting[j].SetPagingFilesSetting(CDriveSetting::eNoPagingFile, 0, 0);
				}
			}
			else {
				for (size_t j=0; j<_vDriveSetting.size(); ++j) {
					if (_vDriveSetting[j].GetDriveLetter()[0] == vsPageFiles[i][0]) {
						ofs << "Drive: " << narrow(vsPageFiles[i]) << " ";
						_vDriveSetting[j].SetPagingFilesSetting(vsPageFiles[i]);
						CDriveSetting::EPagingSizeType n1;
						unsigned int n2, n3;
						boost::tie(n1,n2,n3) = _vDriveSetting[j].GetPagingFileStatus();
					}
				}
			}
		}
		HWND hComboDrive = GetDlgItem(_hDlg, IDC_COMBO_DRIVE);
		SendMessage(hComboDrive, CB_RESETCONTENT, 0, 0);
		for (size_t j=0; j<_vDriveSetting.size(); ++j) {
			string s1 = narrow(_vDriveSetting[j].GetDriveLetter());
			int n=SendMessage(hComboDrive, CB_INSERTSTRING, j, (LPARAM)(_vDriveSetting[j].GetDriveLetter().c_str()));
		}
		SendMessage(hComboDrive, CB_SETCURSEL, 0, 0);
		ofs << "OK" << endl;

		//disk init
		ofs << "Disk Setup ";
		HWND hComboDisk = GetDlgItem(_hDlg, IDC_COMBO_DISK);
		_vDiskSetting.clear();
		SendMessage(hComboDisk, CB_RESETCONTENT, 0, 0);
		std::vector<SDiskState> vDiskState;
		GetDiskState(vDiskState);
		ofs << "DiskNum: " << vDiskState.size() << " ";
		for (size_t i=0; i<vDiskState.size(); ++i) {
			CDiskSetting DiskSetting;
			ofs << " " << narrow(vDiskState[i].sModel) << " (id=" << vDiskState[i].nPhysicalDriveID << ")";
			if (DiskSetting.OpenDrive(vDiskState[i].nPhysicalDriveID, vDiskState[i].sModel)) {
				_vDiskSetting.push_back(DiskSetting);
				ofs << " opened ";
			}
			else {
				ofs << " failed ";
				//USBhCuȂǂ͎s
//				throw (exception("OpenDisk Failed!"));
			}
		}
		ofs << " OK" << endl;
		SendMessage(hComboDisk, CB_RESETCONTENT, 0, 0);
		for (size_t i=0; i<_vDiskSetting.size(); ++i) {
			int n=SendMessage(hComboDisk, CB_INSERTSTRING, i, (LPARAM)(_vDiskSetting[i].GetDriveName().c_str()));
		}
		SendMessage(hComboDisk, CB_SETCURSEL, 0, 0);
		if (_vDiskSetting.empty()) {
			EnableWindow(GetDlgItem(hDlg, IDC_COMBO_DISK), false);
			EnableWindow(GetDlgItem(hDlg, IDC_CHECK_WRITECACHE), false);
			EnableWindow(GetDlgItem(hDlg, IDC_CHECK_READCACHE), false);
		}

		//service
		ofs << "Service Setup " << endl;
		OpenServices(ofs);

		//drive/disk status
		SetDriveStatusToGUI();
		SetDiskStatusToGUI();
	}
	catch (std::exception &e) {
		MessageBox(hDlg, widen(e.what()).c_str(), _T("SSDSetting Error"), MB_OK);
		ofs << endl << e.what() << endl;
		return false;
	}

	AddToolTips();

	ofs << "Initialize Succeeded!" << endl;
	return true;
}

bool CSettingForSSD::ApplySetting() {

	if (!ReadDriveStatusFromGUI()) return false;
	ReadDiskStatusFromGUI();

	wfstream ofs("JSOptimizer_log.txt", std::ios::out | std::ios::app);
	bool bChanged = false;
	wstringstream osErrors;
	try {
		if (!ofs.is_open()) throw (exception("Log File Open Failed!"));
		ofs << endl << endl << "SettingForSSD Apply: " << boost::posix_time::second_clock::local_time() << endl;

		ofs << "Registry Setting " << " ";
		for (size_t i=0; i<_vRegistrySettingWithCheckButton.size(); ++i) {
			bool bState = (Button_GetCheck(_vRegistrySettingWithCheckButton[i].first) == BST_CHECKED);
			ofs << i << " ";
			if (bState != _vRegistrySettingWithCheckButton[i].second->GetStatus()) {
				if (!_vRegistrySettingWithCheckButton[i].second->SetStatus(bState)) {
					ofs << endl << "Registry Setting Failed : "  << _vRegistrySettingWithCheckButton[i].second->GetPos() << "\\" << _vRegistrySettingWithCheckButton[i].second->GetKey() << endl;
					osErrors << "Registry Setting Failed : " << _vRegistrySettingWithCheckButton[i].second->GetPos() << "\\" << _vRegistrySettingWithCheckButton[i].second->GetKey() << endl;
//					throw (exception("Registry Setting Failed"));
				}
				else bChanged = true;
			}
		}
		ofs << " OK" << endl;

		ofs << "Temp Folders Setting " ;
		wchar_t sBuf[256];
		for (size_t i=0; i<_vTEMPFolders.size(); ++i) {
			GetDlgItemText(_hDlg, _vTEMPFolders[i].first, sBuf, 256);
			ofs << sBuf << " ";
			if (wcslen(sBuf) == 0) {
				dout << "remove " << i << endl;
				_vTEMPFolders[i].second->RemoveValue();
			}
			else {
				const CRegistrySetting::ValueType &rValue = _vTEMPFolders[i].second->GetCurrentValue();
				if ((rValue.which() != 1) || (wstring(sBuf) != boost::get<wstring>(rValue))) {
					if (!_vTEMPFolders[i].second->SetValue(wstring(sBuf))) {
						ofs << endl << "Temp Folders Setting Failed : " << _vTEMPFolders[i].second->GetPos() << "\\" << _vTEMPFolders[i].second->GetKey() << endl;
						osErrors << "Temp Folders Setting Failed : " << _vTEMPFolders[i].second->GetPos() << "\\" << _vTEMPFolders[i].second->GetKey() << endl;
	//					throw (exception("Temp Folders Setting Failed"));
					}
					else bChanged = true;
				}
			}
		}
		ofs << " OK" << endl;

		ofs << "PageFile Setting ";
		vector<wstring> vString;
		if (Button_GetCheck(GetDlgItem(_hDlg, IDC_CHECK_AUTOPAGING)) == BST_CHECKED) {
			vString.push_back(_T("?:\\pagefile.sys"));
		}
		else {
			for (size_t i=0; i<_vDriveSetting.size(); ++i){
				wstringstream ss;
				CDriveSetting::EPagingSizeType eType;
				unsigned int nPageFirst;
				unsigned int nPageMax;
				boost::tie(eType, nPageFirst, nPageMax) = _vDriveSetting[i].GetPagingFileStatus();
				if (eType == CDriveSetting::eSystem) {
					ss << _vDriveSetting[i].GetDriveLetter() << "pagefile.sys " << 0 << " " << 0;
					ofs << ss.str() << " ";
					vString.push_back(ss.str());
				}
				else if (eType == CDriveSetting::eCustom) {
					ss << _vDriveSetting[i].GetDriveLetter() << "pagefile.sys " << nPageFirst << " " << nPageMax;
					ofs << ss.str() << " ";
					vString.push_back(ss.str());
				}
			}
		}
		if (boost::get<std::vector<std::wstring> >(_PageFileSetting.GetCurrentValue()) != vString) {
			if (!_PageFileSetting.SetValue(vString)) {
				throw (exception("PageFile Setting Failed"));
				ofs << endl << "PageFile Setting Failed: "<< endl;
				osErrors << "PageFile Setting Failed: " << endl;
				for (size_t i=0; i<vString.size(); ++i) {
					ofs << vString[i] << " ";
					osErrors << vString[i] << " ";
				}
				ofs << endl;
				osErrors << endl;
			}
			else bChanged = true;	
		}
		else {
		}
		ofs << " OK" << endl;

		ofs << "Disk Setting ";
		for (size_t i=0; i<_vDiskSetting.size(); ++i){
			ofs << _vDiskSetting[i].GetDriveName() << " ";
			int nRes = _vDiskSetting[i].ApplySetting();
			if (nRes == 0) {
				ofs << endl << "DiskSetting Faild: " << _vDiskSetting[i].GetDriveName() << endl;
				osErrors << "DiskSetting Faild: "  << _vDiskSetting[i].GetDriveName() << endl;
//				throw (exception("Disk Setting Failed"));
			}
			if (nRes == 2) bChanged = true;
		}
		ofs << " OK" << endl;

		//service
		ofs << "Service Setting " ;
		if (ApplyServices(ofs, osErrors)) {
			bChanged = true;
		}

		ofs << "Setting Finished" << endl;
		if (osErrors.str().empty()) {
			if (bChanged) {
				MessageBox(_hDlg, _T("ݒɐ܂BύX𔽉f邽߂ɂ́AċNKvłB"), _T("JSOptimizer Succeeded!"), MB_OK);
			}
			else {
//				MessageBox(_hDlg, _T("ݒɐ܂B"), _T("JSOptimizer Succeeded!"), MB_OK);
			}
		}
		else {
			osErrors << "ȏ̐ݒɎs܂BoO񍐂肢܂B" << endl;
			MessageBox(_hDlg, osErrors.str().c_str(), _T("JSOptimizer Failed!"), MB_OK);
		}
	}
	catch (std::exception &e) {
		MessageBox(_hDlg, widen(e.what()).c_str(), _T("SSDSetting Error"), MB_OK);
		ofs << endl << e.what() << endl;
		return false;
	}
	return true;
}


bool CSettingForSSD::ApplyServices(std::wfstream &ofs, std::wstringstream& osErrors) {

	SC_HANDLE hSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_CONNECT);
	if (hSCManager == NULL) {
		throw (exception("OpenSCManager Failed"));
	}
	bool bRet = false;
	for (size_t i=0; i<_vServices.size(); ++i) {
		SC_HANDLE hService = _vServices[i].get<0>();
		wstring sServiceName = _vServices[i].get<1>();
		HWND hCombo = _vServices[i].get<2>();

		DWORD dwSize;
		BOOL b1 = QueryServiceConfig(hService, NULL, 0, &dwSize);
		QUERY_SERVICE_CONFIG *lpServiceConfig = (LPQUERY_SERVICE_CONFIG)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwSize);
		BOOL b2 = QueryServiceConfig(hService, lpServiceConfig, dwSize, &dwSize);
		if (b2) {
			int nSelected = SendMessage(hCombo, CB_GETCURSEL, 0, 0);
			if ( ((nSelected == 0) && (lpServiceConfig->dwStartType != SERVICE_AUTO_START)) 
				|| ((nSelected == 1) && (lpServiceConfig->dwStartType != SERVICE_DEMAND_START)) 
				|| ((nSelected == 2) && (lpServiceConfig->dwStartType != SERVICE_DISABLED)) ) 
			{
				if (nSelected == 0) lpServiceConfig->dwStartType = SERVICE_AUTO_START;
				if (nSelected == 1) lpServiceConfig->dwStartType = SERVICE_DEMAND_START;
				if (nSelected == 2) lpServiceConfig->dwStartType = SERVICE_DISABLED;
				if (!ChangeServiceConfig(hService, SERVICE_NO_CHANGE, lpServiceConfig->dwStartType, SERVICE_NO_CHANGE,
						NULL, NULL, NULL, NULL, NULL, NULL, lpServiceConfig->lpDisplayName)) 
				{
					ofs << endl << sServiceName << " Change Service failed : " << GetLastErrorString(GetLastError()) << endl;
					osErrors << sServiceName << " Change Service failed : " << GetLastErrorString(GetLastError()) << endl;
				}
				else  bRet = true;
			}
		}
		else {
			ofs << endl << sServiceName << " QueryService failed : " << GetLastErrorString(GetLastError()) << endl;
			osErrors << sServiceName << " QueryService failed : " << GetLastErrorString(GetLastError()) << endl;
		}
		HeapFree(GetProcessHeap(), 0, lpServiceConfig);
		ofs << " OK" << endl;
	}
	CloseServiceHandle(hSCManager);

	return bRet;
}