#pragma once
#include <windows.h>
#include <boost/shared_ptr.hpp>
#include <boost/variant.hpp>
#include <vector>
#include "RegistrySetting.h"
#include <sstream>
#include <map>
#include <boost/tuple/tuple.hpp>

class CDriveSetting {
public:
	enum EPagingSizeType {
		eNoPagingFile, eSystem, eCustom
	};

	CDriveSetting(const std::wstring& sDriveLetter);
	~CDriveSetting(){}

	const std::wstring& GetDriveLetter() {return _sDriveLetter;}
	bool SetPagingFilesSetting(const std::wstring& sStr);
	bool SetPagingFilesSetting(EPagingSizeType eType, unsigned int nFirst, unsigned int nMax);
	boost::tuple<EPagingSizeType, unsigned int, unsigned int> GetPagingFileStatus() const {return boost::make_tuple(_nPagingFileStatus, _nPageSizeFirst, _nPageSizeMax);}
	
protected:
	std::wstring _sDriveLetter;
	EPagingSizeType _nPagingFileStatus;
	unsigned int _nPageSizeFirst;
	unsigned int _nPageSizeMax;
	bool _bIndexing;   // 
};

class CDiskSetting {
public:
	CDiskSetting() {}
	~CDiskSetting() {}

	bool OpenDrive(int nDriveNumber, const std::wstring &rsDriveName);
	std::pair<bool, bool> GetChacheStatus() const {return std::make_pair(_bWriteCache, _bReadCache);}
	void SetCacheStatusSetting(bool bWrite, bool bRead);
	int ApplySetting();
	const std::wstring& GetDriveName() const {return _sDriveName;}

protected:
	int _nDriveNumber;
	std::wstring _sDriveName;
	bool _bWriteCache;
	bool _bReadCache;
};


class CSettingForSSD
{
public:
	CSettingForSSD();
	virtual ~CSettingForSSD();

	INT_PTR CALLBACK Proc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
	bool Init(HWND hDlg);
	bool ApplySetting();

protected:

	void ReadDiskStatusFromGUI();
	bool ReadDriveStatusFromGUI();
	void SetDiskStatusToGUI();
	void SetDriveStatusToGUI();

	bool AddRegistrySetting(int nDlgItem, HKEY hRoot, const std::wstring& sPos, const std::wstring& sKey, DWORD nType, 
													const boost::variant<DWORD,std::wstring> &valOKState, const boost::variant<DWORD,std::wstring> &valNGState, bool bNotExistState);
	bool ChangeTempFolder(unsigned int nID);
	void DisableAllPagingFile();
	void AddToolTips();

	std::vector<std::pair<HWND, boost::shared_ptr<CRegistrySettingWithCheckButton> > > _vRegistrySettingWithCheckButton;
	std::vector<std::pair<int, boost::shared_ptr<CRegistrySetting>> > _vTEMPFolders;
	std::vector<CDriveSetting> _vDriveSetting;
	std::vector<CDiskSetting> _vDiskSetting;
	CRegistrySetting _PageFileSetting;

	std::wstring _sUserTEMPDir;
	std::wstring _sUserTMPDir;
	std::wstring _sSystemTEMPDir;
	std::wstring _sSystemTMPDir;

	HWND _hDlg;
	int _nCurrentSelectedDrive;
	int _nCurrentSelectedDisk;

	std::vector< boost::tuple<SC_HANDLE, std::wstring, HWND> > _vServices;
	void OpenServices(std::ofstream &ofs);
	bool ApplyServices(std::wfstream &ofs, std::wstringstream& osErrors);

};

void InitSSDSetting();
INT_PTR CALLBACK SSDSettingProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
