#include "SSDSmartUtil.h"
#include <tchar.h>
#include <iomanip>
#include <iostream>
#include <fstream>
#include <wbemcli.h>
#include <winsvc.h>
#include "DiskState.h"
#include "OutputDebugStream.h"

#pragma comment(lib, "wbemuuid.lib")
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

using namespace std;

bool	IsServiceRunning(LPCTSTR pszName)
{
	bool			ret;
	BOOL			bRet;
	SC_HANDLE		hManager;
	SC_HANDLE		hService;
	SERVICE_STATUS	sStatus;

	ret = false;
	hManager = NULL;
	hService = NULL;
	while(1) {
		hManager = ::OpenSCManager(NULL,NULL,GENERIC_EXECUTE);
		if(hManager == NULL)
			break;

		hService = ::OpenService(hManager,pszName,SERVICE_QUERY_STATUS);
		if(hService == NULL)
			break;

		::ZeroMemory(&sStatus,sizeof(SERVICE_STATUS));
		bRet = ::QueryServiceStatus(hService,&sStatus);
		if(bRet == FALSE)
			break;

		if(sStatus.dwCurrentState == SERVICE_RUNNING)
			ret = true;
		break;
	}
	if(hService)
		::CloseServiceHandle(hService);
	if(hManager)
		::CloseServiceHandle(hManager);
	return	ret;
}


bool	EasyStart(LPCTSTR pszName)
{
	bool			ret;
	BOOL			bRet;
	SC_HANDLE		hManager;
	SC_HANDLE		hService;
	SERVICE_STATUS	sStatus;

	ret = false;
	hManager = NULL;
	hService = NULL;
	while(1) {
		hManager = ::OpenSCManager(NULL,NULL,GENERIC_EXECUTE);
		if(hManager == NULL) break;

		hService = ::OpenService(hManager,pszName,SERVICE_START | SERVICE_QUERY_STATUS);
		if(hService == NULL) break;

		::ZeroMemory(&sStatus,sizeof(SERVICE_STATUS));
		bRet = ::QueryServiceStatus(hService,&sStatus);
		if(bRet == FALSE) break;

		if(sStatus.dwCurrentState == SERVICE_RUNNING) {
			ret = true;
			break;
		}
		if(sStatus.dwCurrentState == SERVICE_STOPPED) {
			bRet = ::StartService(hService,NULL,NULL);
			int count = 0;
			while(::QueryServiceStatus(hService,&sStatus))
			{
				// [v (ő 5 b WMI ̏҂)
				if(count >= 10) break;
				if(sStatus.dwCurrentState == SERVICE_RUNNING) {
					ret = true;
					break;
				}
				::Sleep(500);
				count++;
				continue;
			}
		}
		break;
	}
	if(hService) ::CloseServiceHandle(hService);
	if(hManager) ::CloseServiceHandle(hManager);
	return	ret;
}

SDiskState GetDiskStateFromWbem(IWbemClassObject*	pCOMDev) {
	wstring model, deviceId, diskSize;
	INT physicalDriveId = -1, scsiPort = -1, scsiTargetId = -1;

	SDiskState Result;
	memset(&Result, 0, sizeof(SDiskState));
	wchar_t* pcStopped;

	VARIANT pVal;
	VariantClear(&pVal);
	if(pCOMDev->Get(L"Size", 0L, &pVal, NULL, NULL) == WBEM_S_NO_ERROR && pVal.vt > VT_NULL)
	{
		diskSize = pVal.bstrVal;
		VariantClear(&pVal);
		Result.nDiskSize =  _wcstoi64(diskSize.c_str(), &pcStopped, 10);
	}
	if(pCOMDev->Get(L"DeviceID", 0L, &pVal, NULL, NULL) == WBEM_S_NO_ERROR && pVal.vt > VT_NULL)
	{
		deviceId = pVal.bstrVal;
		size_t len = deviceId.size();
		if(_ttoi(deviceId.c_str()+len-2) >= 10)
		{
			physicalDriveId = _ttoi(deviceId.c_str()+len-2);
		}
		else
		{
			physicalDriveId = _ttoi(deviceId.c_str()+len-1);
		}
//		wcout << deviceId.c_str() << _T(" ") << physicalDriveId << endl;
		Result.nPhysicalDriveID = physicalDriveId;
		VariantClear(&pVal);
	}
	if(pCOMDev->Get(L"Model", 0L, &pVal, NULL, NULL) == WBEM_S_NO_ERROR && pVal.vt > VT_NULL)
	{
		Result.sModel = pVal.bstrVal;
		VariantClear(&pVal);
	}
	if(pCOMDev->Get(L"SCSIPort", 0L, &pVal, NULL, NULL) == WBEM_S_NO_ERROR && pVal.vt > VT_NULL)
	{
		Result.nPort = pVal.intVal;
		VariantClear(&pVal);
	}			
	if(pCOMDev->Get(L"SCSITargetId", 0L, &pVal, NULL, NULL) == WBEM_S_NO_ERROR && pVal.vt > VT_NULL)
	{
		Result.nTargetID = pVal.intVal;
		VariantClear(&pVal);
	}
	SAFE_RELEASE(pCOMDev);

	return Result;
}

bool GetDiskState(std::vector<SDiskState> &rvDiskState)  {

	HRESULT hRes = S_OK;

	IWbemLocator*			pIWbemLocator = NULL;
	IWbemServices*			pIWbemServices = NULL;
	IEnumWbemClassObject*	pEnumCOMDevs = NULL;
	IEnumWbemClassObject*	pEnumCOMDevs2 = NULL;

	static bool bInitialized = false;
	static vector<SDiskState> vDiskState;

	if (!bInitialized) {

		bool initWmi = true;
		for(int i = 0; i < 3; i++) {
			if(!IsServiceRunning(_T("Winmgmt"))) {
				initWmi = EasyStart(_T("Winmgmt"));
				continue;
			}
			else break;
		}
		bool IsEnabledWmi = false;
		if (!initWmi) return false;
		try {
			CoInitialize(NULL);
			CoInitializeSecurity(NULL, -1, NULL, NULL, RPC_C_AUTHN_LEVEL_DEFAULT,
				RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE, NULL);
			if(FAILED(CoCreateInstance(CLSID_WbemLocator, NULL, CLSCTX_INPROC_SERVER,
				IID_IWbemLocator, (LPVOID *)&pIWbemLocator))) {
				CoUninitialize();
			}
			else  {
				long securityFlag = 0;
				OSVERSIONINFOEX osvi;
				osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
				if (GetVersionEx((LPOSVERSIONINFO)&osvi) && 
					 (osvi.dwMajorVersion >= 6 || (osvi.dwMajorVersion == 5 && osvi.dwMinorVersion >= 1)))
					securityFlag = WBEM_FLAG_CONNECT_USE_MAX_WAIT;
				if(FAILED(pIWbemLocator->ConnectServer(SysAllocString(L"\\\\.\\root\\cimv2"), 
					NULL, NULL, 0L,
					securityFlag,
					NULL, NULL, &pIWbemServices)))
				{
					CoUninitialize();
				}
				else {
					hRes = CoSetProxyBlanket(pIWbemServices, RPC_C_AUTHN_WINNT, RPC_C_AUTHZ_NONE,
						NULL, RPC_C_AUTHN_LEVEL_CALL, RPC_C_IMP_LEVEL_IMPERSONATE, NULL, EOAC_NONE);
					if(FAILED(hRes)) {
						goto wmi_end;
					}
					else {
						IsEnabledWmi = true;
					}
				}
			}
			pIWbemServices->ExecQuery(SysAllocString(L"WQL"), 
				SysAllocString(L"SELECT * FROM Win32_DiskDrive"), WBEM_FLAG_FORWARD_ONLY | WBEM_FLAG_RETURN_IMMEDIATELY, NULL, &pEnumCOMDevs);
			ULONG uReturned = 1;
			IWbemClassObject* pCOMDev = NULL;
			while(pEnumCOMDevs && SUCCEEDED(pEnumCOMDevs->Next(10000, 1, &pCOMDev, &uReturned)) && uReturned == 1)
			{
				vDiskState.push_back(GetDiskStateFromWbem(pCOMDev));
			}
		}
		catch(...)
		{
			dout << "GetDiskState Failed!" << endl;
//			return false;
		}

	wmi_end:

		SAFE_RELEASE(pEnumCOMDevs);
		SAFE_RELEASE(pEnumCOMDevs2);
		SAFE_RELEASE(pIWbemServices);
		SAFE_RELEASE(pIWbemLocator);
		CoUninitialize();

		if (IsEnabledWmi) {
			rvDiskState = vDiskState;
			bInitialized = true;
		}
		return IsEnabledWmi;
	}

	rvDiskState = vDiskState;
	return true;

}
