#include "SSDSmartUtil.h"
#include "SSDSmart.h"
#include "OutputDebugStream.h"
#include <iomanip>
#include <math.h>
#include <boost/regex.hpp>
#include <fstream>
#include <wbemcli.h>
#include <iostream>
#include <winsvc.h>
#include "DiskState.h"
#include "DnpATAPI.h"
#include <tchar.h>

//#pragma comment(lib, "wbemuuid.lib")
//#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

using namespace std;

CDnpATAPI *CSSDStatus::_pDnpATAPI = NULL;
std::vector<CJMFStatus::SFlashInfo> CJMFStatus::_vFlashInfo;

bool CJMFStatus::ReadFlashData(const std::wstring& rsFlashListFileName) {

	ifstream fFlashListFile(rsFlashListFileName.c_str());
	if (!fFlashListFile.is_open()) return false;

	string sTemp;
	while (getline(fFlashListFile, sTemp)) {
		SFlashInfo CurrentInfo;
		istringstream ss(sTemp);
		for (size_t i=0; i<5; ++i) {
			CurrentInfo.aID[i] = 0;
			int n;
			ss >> showbase >> hex >> n;
			CurrentInfo.aID[i] = (unsigned char)n;
		}
		ss >> dec >> CurrentInfo.sVendor >> CurrentInfo.nType;
		string sTemp2; double d;
		while (ss >> sTemp2 >> d) {
			CurrentInfo.vCandidates.push_back(make_pair(sTemp2, d));
		}
		_vFlashInfo.push_back(CurrentInfo);
	}

	return true;
}


bool CSSDStatus::Init(const SDiskState* pDiskState) {

	nDeviceNo = pDiskState->nPhysicalDriveID;
	CDnpATAPI::IDENTIFY_DEVICE Data;
	CDnpATAPI::CModelInfo Info;
	if (!_pDnpATAPI->GetIdentifyDevice(nDeviceNo, &Data, &Info)) return false;
	CDnpATAPI::CSmartAttribute	cAttrInfo;
	if (!_pDnpATAPI->GetSmartReadData(nDeviceNo, &cAttrInfo)) return false;

	sModelName = Info._strModel;
	sFirmwareVersion = Info._strFirmware;
	sSerialNo = Info._strSerialNo;
	nDriveSize = pDiskState->nDiskSize;
	nFlashType = CJMFStatus::eUnknown;

	return InitImpl(cAttrInfo);
}

bool CSSDStatus::Update() {

	CDnpATAPI::CSmartAttribute	cAttrInfo;
	if (!_pDnpATAPI->GetSmartReadData(nDeviceNo, &cAttrInfo)) return false;
	tMeasuredTime = boost::posix_time::second_clock::local_time();
	return UpdateImpl(cAttrInfo);
}


bool CJMFStatus::InitImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo) {

	BYTE cbAttrID;
	BYTE pcbData[9];
	int nSize;
	if (!rAttrInfo.GetAttributeCount(&nSize)) return false;
	for(int i = 0; i<nSize; ++i) {
		if(!rAttrInfo.GetAttribute(i,&cbAttrID,pcbData)) break;

		if (cbAttrID == 229) {//E5 FLASHID
			memcpy(anFlashID, pcbData+1, 8);
		}
		if (cbAttrID == 232) {//E8 Firmware version information Table
			sFirmwareDate.resize(8);
			sFirmwareDate[0] = pcbData[1];
			sFirmwareDate[1] = pcbData[2];
			sFirmwareDate[2] = '/';
			sFirmwareDate[3] = pcbData[3];
			sFirmwareDate[4] = pcbData[4];
			sFirmwareDate[5] = '/';
			sFirmwareDate[6] = pcbData[5];
			sFirmwareDate[7] = pcbData[6];
			nChannels = (int)pcbData[7];
			nBanks = (int)pcbData[8];
		}
		if (cbAttrID == 233) {//E9 ECC Fail Record
		}
		if (cbAttrID == 234) {//EA Average Erase Count, Maximum Erase Count
		}
		if (cbAttrID == 235) {//EB Good Block Count, System Block Count
			nGoodBlockCount = 256*256*pcbData[1] + 256*pcbData[2] + pcbData[3];
			nSystemBlockCount = 256*pcbData[4] + pcbData[5];
		}
	}
	nBlockSize = round(nDriveSize/1000.0/1000.0/(nGoodBlockCount-nSystemBlockCount));
	sFlashName   = "Unknown";
	sFlashVendor = "Unknown";

	if (anFlashID[1]==0x2C) sFlashVendor = "Micron";
	if (anFlashID[1]==0xEC) sFlashVendor = "Samsung";
	if (anFlashID[1]==0x98) sFlashVendor = "Toshiba";
	if (anFlashID[1]==0xAD) sFlashVendor = "Hynix";
	if (anFlashID[1]==0x89) sFlashVendor = "Intel";

	for (size_t n=0; n<_vFlashInfo.size(); ++n) {
		const SFlashInfo &rFlashInfo = _vFlashInfo[n];
		if (memcmp(rFlashInfo.aID, anFlashID+1, 5) == 0) {
			sFlashVendor = rFlashInfo.sVendor;
			if (rFlashInfo.nType==0) nFlashType = eMLC;
			if (rFlashInfo.nType==1) nFlashType = eSLC;
			stringstream ss;
			ss << rFlashInfo.vCandidates[0].first;
			for (size_t i=1; i<rFlashInfo.vCandidates.size(); ++i) {
				ss << "/" << rFlashInfo.vCandidates[i].first;
			}
			sFlashName = ss.str();
		}
	}
	return true;
}

bool CJMFStatus::UpdateImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo) {

	BYTE cbAttrID;
	BYTE pcbData[9];
	int nSize;
	if (!rAttrInfo.GetAttributeCount(&nSize)) return false;

	for(int i = 0; i<nSize; ++i) {
		if(!rAttrInfo.GetAttribute(i,&cbAttrID,pcbData)) break;
		if (cbAttrID == 12) {//0C Power Cycle Count 
			memcpy(&nPowerCycleCount, pcbData+2, 4);
		}
		if (cbAttrID == 229) {//E5 FLASHID
		}
		if (cbAttrID == 232) {//E8 Firmware version information Table
		}
		if (cbAttrID == 233) {//E9 ECC Fail Record
			nECCFailCount = pcbData[1]*256+pcbData[2];
			memcpy(anLastECAddress, pcbData+3, 3);
			nLastECChannel = pcbData[6];
			nLastECBank = pcbData[7];
		}
		if (cbAttrID == 234) {//EA Average Erase Count, Maximum Erase Count
			nAverageEraseCount = 256*256*pcbData[1] + 256*pcbData[2] + pcbData[3];
			nMaxEraseCount     = 256*256*pcbData[4] + 256*pcbData[5] + pcbData[6];
		}
		if (cbAttrID == 235) {//EB Good Block Count, System Block Count
			nGoodBlockCount   = 256*256*pcbData[1] + 256*pcbData[2] + pcbData[3];
			nSystemBlockCount = 256*pcbData[4] + pcbData[5];
		}
	}
	return true;
}

bool CIDXStatus::InitImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo) {

	BYTE cbAttrID;
	BYTE pcbData[9];
	int nSize;
	if (!rAttrInfo.GetAttributeCount(&nSize)) return false;
	for(int i = 0; i<nSize; ++i) {
		if(!rAttrInfo.GetAttribute(i,&cbAttrID,pcbData)) break;
	}
	return true;
}

bool CIDXStatus::UpdateImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo) {

	BYTE cbAttrID;
	BYTE pcbData[9];
	int nSize;
	if (!rAttrInfo.GetAttributeCount(&nSize)) return false;
	for(int i = 0; i<nSize; ++i) {
		if(!rAttrInfo.GetAttribute(i,&cbAttrID,pcbData)) break;
		switch(cbAttrID) {
			case   1: memcpy(&nRawReadErrorRate, pcbData, 8); break;
			case   9: memcpy(&nPoweronHours, pcbData, 8); break;
			case  12: memcpy(&nDevicePowerCycleCount, pcbData, 8); break;
			case 184: memcpy(&nInitialBadBlockCount, pcbData, 8); break;
			case 195: memcpy(&nProgramFailureBlockCount, pcbData, 8); break;
			case 196: memcpy(&nEraseFailureBlockCount, pcbData, 8); break;
			case 197: memcpy(&nReadFailureBlockCount, pcbData, 8); break;
			case 198: memcpy(&nTotalCountofReadSectors, pcbData, 8); break;
			case 199: memcpy(&nTotalCountofWriteSectors, pcbData, 8); break;
			case 200: memcpy(&nTotalCountofReadCommands, pcbData, 8); break;
			case 201: memcpy(&nTotalCountofWriteCommands, pcbData, 8); break;
			case 202: memcpy(&nTotalCountofErrorBitsfromFlash, pcbData, 8); break;
			case 203: memcpy(&nTotalCountofReadSectorswithCorrectableBitErrors, pcbData, 8); break;
			case 204: memcpy(&nBadBlockFullFlag, pcbData, 8); break;
			case 205: memcpy(&nMaximumPECountSpecification, pcbData, 8); break;
			case 206: memcpy(&nMinEraseCount, pcbData, 4); break;
			case 207: memcpy(&nMaxEraseCount, pcbData, 4); break;
			case 208: memcpy(&nAverageEraseCount, pcbData, 4); break;
			case 209: memcpy(&nRemainingDriveLife, pcbData, 4); break;
			default: {
				std::map<BYTE, unsigned long long>::iterator it = mUnknownValues.find(cbAttrID);
				unsigned long long n;
				memcpy(&n, pcbData, 8);
				if (it != mUnknownValues.end()) {
					it->second = n;
				}
				else {
					mUnknownValues.insert(make_pair(cbAttrID, n));
				}
				break;
			}
		}
	}
	return true;
}


CSSDSmart::CSSDSmart(void)
{
	Init();
	Update();
}

CSSDSmart::~CSSDSmart(void)
{
	for (size_t i=0; i<_vpSSDStatus.size(); ++i) {
		delete _vpSSDStatus[i];
	}
}

CSSDSmart& CSSDSmart::GetInstance() {
	static CSSDSmart Instance;
	return Instance;
}

CSSDStatus::ESSDType CSSDSmart::GetDriveType(const CDnpATAPI::CSmartAttribute& cAttrInfo) {

	int nSize;
	if (!cAttrInfo.GetAttributeCount(&nSize)) return CSSDStatus::eOther;
	BYTE cbAttrID;
	vector<BYTE> vID;
	for(int i = 0; i<nSize; ++i) {
		if(!cAttrInfo.GetAttribute(i,&cbAttrID,NULL)) break;
		vID.push_back(cbAttrID);
	}

	if ( (find(vID.begin(), vID.end(),  12) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 229) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 232) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 233) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 234) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 235) != vID.end()) )
	{
		return CSSDStatus::eJMF;
	}

	if ( (find(vID.begin(), vID.end(),   1) != vID.end()) &&
			 (find(vID.begin(), vID.end(),   9) != vID.end()) &&
			 (find(vID.begin(), vID.end(),  12) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 184) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 195) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 196) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 197) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 198) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 199) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 200) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 201) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 202) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 203) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 204) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 205) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 206) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 207) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 208) != vID.end()) &&
			 (find(vID.begin(), vID.end(), 209) != vID.end()) )
	{
		return CSSDStatus::eIDX;
	}

	return CSSDStatus::eOther;
}

CSSDStatus*  CSSDSmart::MakeSuitableSSDStatus(CDnpATAPI::CSmartAttribute& rcAttrInfo) {

	CSSDStatus::ESSDType SSDType = GetDriveType(rcAttrInfo);
	CSSDStatus *pStatus = NULL;
	if (SSDType == CSSDStatus::eIDX) {
		pStatus = new CIDXStatus();
	}
	if (SSDType == CSSDStatus::eJMF) {
		pStatus = new CJMFStatus();
	}
	return pStatus;
}

bool CSSDSmart::AddSuitableSSD(const SDiskState* pDiskState) {

	dout << "Drive #" << pDiskState->nPhysicalDriveID << " ";
	CDnpATAPI::CSmartAttribute	cAttrInfo;
	if (!_DnpATAPI.GetSmartReadData(pDiskState->nPhysicalDriveID, &cAttrInfo)) {
		dout << "GetSmartReadData Failed." << endl;
		return false;
	}
	CSSDStatus *pStatus = MakeSuitableSSDStatus(cAttrInfo);
	if (pStatus && (pStatus->Init(pDiskState))){
		_vpSSDStatus.push_back(pStatus);
		dout << "OK. " << endl;
	}
	else {
		dout << "Init Failed." << pDiskState->nPhysicalDriveID << endl;
		delete pStatus;
	}
	return true;
}


void CSSDSmart::Update() {

	for (size_t i=0; i<_vpSSDStatus.size(); ++i) {
		_vpSSDStatus[i]->Update();
	}
}

void CSSDSmart::Init() {

	wstring _sFlashListFileName = GetModulePath();
	_sFlashListFileName.append(_T("\\FlashData.dat"));
	CJMFStatus::ReadFlashData(_sFlashListFileName);

	CSSDStatus::_pDnpATAPI = &_DnpATAPI;
	_vDiskState.clear();
	if (GetDiskState(_vDiskState)) {
		for (size_t i=0; i<_vDiskState.size(); ++i) {
			AddSuitableSSD(&(_vDiskState[i]));
		}
	}
	else {
		dout << "InitWmi failed!" << endl;
	}
	dout << endl;
	Update();
}
