#pragma once

#include "DnpATAPI.h"
#include <vector>
#include "DiskState.h"
#include <map>
#include <boost/date_time/posix_time/posix_time.hpp>

class CSSDStatus {

public:
	CSSDStatus() {}
	virtual ~CSSDStatus() {}

	//const
	unsigned int nDeviceNo;   //PhysicalDevice No
	enum ESSDType {
		eOther, eJMF, eIDX
	};
	ESSDType nSSDType;
	std::string sModelName;
	std::string sSerialNo;
	std::string sFirmwareVersion;
	unsigned long long nDriveSize; //[Byte]
	enum EFlashType {
		eUnknown, eMLC, eSLC
	};
	EFlashType nFlashType;  //0=MLC, 1=SLC, 2=Unknown

	//non-const
	unsigned int nAverageEraseCount;
	unsigned int nMaxEraseCount;
	boost::posix_time::ptime tMeasuredTime;

	bool Init(const SDiskState* pDiskState);
	bool Update();

	static CDnpATAPI *_pDnpATAPI;
protected:
	virtual bool InitImpl  (const CDnpATAPI::CSmartAttribute &rAttrInfo) = 0;
	virtual bool UpdateImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo) = 0;
};

class CJMFStatus : public CSSDStatus{

public:
	CJMFStatus() : CSSDStatus() {
		nSSDType = eJMF;
	}
	virtual ~CJMFStatus () {}

	//const
	std::string sFirmwareDate;
	unsigned int nChannels;
	unsigned int nBanks;
	int nBlockSize; //[MB]
	std::string sFlashName;
	std::string sFlashVendor;
	BYTE anFlashID[8];

	//non-const
	unsigned int nPowerCycleCount;
	unsigned int nGoodBlockCount;
	unsigned int nSystemBlockCount;
	unsigned int nECCFailCount;
	unsigned int nLastECChannel;
	unsigned int nLastECBank;
	BYTE anLastECAddress[3];

	struct SFlashInfo {
		unsigned char aID[5];
		int nType; //0=MLC, 1=SLC
		std::string sVendor;
		std::vector<std::pair<std::string, double> > vCandidates;
	};
	static bool ReadFlashData(const std::wstring& rsFlashListFileName);
	static std::vector<SFlashInfo> _vFlashInfo;

protected:
	virtual bool InitImpl  (const CDnpATAPI::CSmartAttribute &rAttrInfo);
	virtual bool UpdateImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo);
};

class CIDXStatus : public CSSDStatus {

public:
	CIDXStatus() : CSSDStatus() {
		nSSDType = eIDX;
	}
	virtual ~CIDXStatus () {}

	unsigned long long nRawReadErrorRate;
	unsigned long long nPoweronHours;
	unsigned long long nDevicePowerCycleCount;
	unsigned long long nInitialBadBlockCount;
	unsigned long long nProgramFailureBlockCount;
	unsigned long long nEraseFailureBlockCount;
	unsigned long long nReadFailureBlockCount;
	unsigned long long nTotalCountofReadSectors;
	unsigned long long nTotalCountofWriteSectors;
	unsigned long long nTotalCountofReadCommands;
	unsigned long long nTotalCountofWriteCommands;
	unsigned long long nTotalCountofErrorBitsfromFlash;
	unsigned long long nTotalCountofReadSectorswithCorrectableBitErrors;
	unsigned long long nBadBlockFullFlag;
	unsigned long long nMaximumPECountSpecification;
	unsigned int nRemainingDriveLife;
	unsigned int nMinEraseCount;
	std::map<BYTE, unsigned long long> mUnknownValues;

protected:
	virtual bool InitImpl  (const CDnpATAPI::CSmartAttribute &rAttrInfo);
	virtual bool UpdateImpl(const CDnpATAPI::CSmartAttribute &rAttrInfo);
};

class CSSDSmart
{
public:
	static CSSDSmart &GetInstance();
	~CSSDSmart(void);

	void Update();
	size_t GetSuitableDriveNum() const {
		return _vpSSDStatus.size();
	}
	const std::vector<CSSDStatus*> &GetSSDStatus() const {return _vpSSDStatus;}

protected:

	CSSDSmart(void);
	CSSDSmart(const CSSDSmart &r);//Singleton
	CSSDSmart &operator=(const CSSDSmart &r);//Singleton

	void Init();
	CDnpATAPI	_DnpATAPI;

	std::vector<CSSDStatus*> _vpSSDStatus;
	std::vector<SDiskState> _vDiskState;
	CSSDStatus::ESSDType GetDriveType (const CDnpATAPI::CSmartAttribute& rAttribute);

	bool AddSuitableSSD(const SDiskState* pDiskState);
	virtual CSSDStatus* MakeSuitableSSDStatus(CDnpATAPI::CSmartAttribute& rcAttrInfo);
};
